@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2011, Thorvald Natvig <thorvald@natvig.com>
 
    All rights reserved.
 
@@ -28,18 +28,21 @@
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
-#include <QtCore>
+#include "murmur_pch.h"
+
 #include "Timer.h"
 
-Timer::Timer() {
-	uiStart = now();
+Timer::Timer(bool start) {
+	uiStart = start ? now() : 0;
 }
 
 quint64 Timer::elapsed() const {
+	Q_ASSERT(uiStart != 0);
 	return now() - uiStart;
 }
 
 bool Timer::isElapsed(quint64 us) {
+	Q_ASSERT(uiStart != 0);
 	if (elapsed() > us) {
 		uiStart += us;
 		return true;
@@ -54,6 +57,18 @@ quint64 Timer::restart() {
 	return e;
 }
 
+bool Timer::isStarted() const {
+	return uiStart != 0;
+}
+
+bool Timer::operator<(const Timer &other) const {
+	return uiStart > other.uiStart;
+}
+
+bool Timer::operator>(const Timer &other) const {
+	return uiStart < other.uiStart;
+}
+
 #if defined(Q_OS_WIN)
 #include <windows.h>
 
