@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2009, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2011, Thorvald Natvig <thorvald@natvig.com>
 
    All rights reserved.
 
@@ -28,12 +28,17 @@
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
-#include "ALSAAudio.h"
-#include "User.h"
-#include "Global.h"
-#include "MainWindow.h"
+#include "mumble_pch.hpp"
+
+#include <alsa/asoundlib.h>
 #include <sys/poll.h>
 
+#include "ALSAAudio.h"
+
+#include "MainWindow.h"
+#include "Global.h"
+#include "User.h"
+
 #define NBLOCKS 8
 
 class ALSAEnumerator {
@@ -94,12 +99,9 @@ void ALSAInit::initialize() {
 
 void ALSAInit::destroy() {
 	QMutexLocker qml(&qmALSA);
-	if (pairALSA)
-		delete pairALSA;
-	if (paorALSA)
-		delete paorALSA;
-	if (cards)
-		delete cards;
+	delete pairALSA;
+	delete paorALSA;
+	delete cards;
 }
 
 ALSAAudioInputRegistrar::ALSAAudioInputRegistrar() : AudioInputRegistrar(QLatin1String("ALSA"),5) {
@@ -302,7 +304,8 @@ ALSAAudioInput::~ALSAAudioInput() {
 	wait();
 }
 
-#define ALSA_ERRBAIL(x) if (!bOk) {} else if ((err=(x)) != 0) { bOk = false; qWarning("ALSAAudio: %s: %s", #x, snd_strerror(err));}
+#define ALSA_ERRBAIL(x) if (!bOk) {} else if ((err=(x)) < 0) { bOk = false; qWarning("ALSAAudio: %s: %s", #x, snd_strerror(err));}
+#define ALSA_ERRCHECK(x) if (!bOk) {} else if ((err=(x)) < 0) {qWarning("ALSAAudio: Non-critical: %s: %s", #x, snd_strerror(err));}
 
 void ALSAAudioInput::run() {
 	QMutexLocker qml(&qmALSA);
@@ -324,7 +327,7 @@ void ALSAAudioInput::run() {
 	snd_pcm_hw_params_alloca(&hw_params);
 
 	ALSA_ERRBAIL(snd_pcm_open(&capture_handle, device_name.data(), SND_PCM_STREAM_CAPTURE, 0));
-	ALSA_ERRBAIL(snd_pcm_hw_params_any(capture_handle, hw_params));
+	ALSA_ERRCHECK(snd_pcm_hw_params_any(capture_handle, hw_params));
 	ALSA_ERRBAIL(snd_pcm_hw_params_set_access(capture_handle, hw_params, SND_PCM_ACCESS_RW_INTERLEAVED));
 	ALSA_ERRBAIL(snd_pcm_hw_params_set_format(capture_handle, hw_params, SND_PCM_FORMAT_S16));
 	ALSA_ERRBAIL(snd_pcm_hw_params_set_rate_near(capture_handle, hw_params, &rrate, NULL));
@@ -342,8 +345,6 @@ void ALSAAudioInput::run() {
 	ALSA_ERRBAIL(snd_pcm_hw_params_current(capture_handle, hw_params));
 	ALSA_ERRBAIL(snd_pcm_hw_params_get_channels(hw_params, &iMicChannels));
 	ALSA_ERRBAIL(snd_pcm_hw_params_get_rate(hw_params, &iMicFreq, NULL));
-	eMicFormat = SampleShort;
-	initializeMixer();
 
 #ifdef ALSA_VERBOSE
 	snd_output_t *log;
@@ -365,6 +366,9 @@ void ALSAAudioInput::run() {
 		return;
 	}
 
+	eMicFormat = SampleShort;
+	initializeMixer();
+
 	char inbuff[wantPeriod * iChannels * sizeof(short)];
 
 	qml.unlock();
@@ -420,7 +424,6 @@ void ALSAAudioOutput::run() {
 	int err = 0;
 	bool bOk = true;
 
-	iMixerFreq = SAMPLE_RATE;
 
 	snd_pcm_hw_params_t *hw_params = NULL;
 	snd_pcm_sw_params_t *sw_params = NULL;
@@ -430,12 +433,12 @@ void ALSAAudioOutput::run() {
 	snd_pcm_sw_params_alloca(&sw_params);
 
 	ALSA_ERRBAIL(snd_pcm_open(&pcm_handle, device_name.data(), SND_PCM_STREAM_PLAYBACK, 0));
-	ALSA_ERRBAIL(snd_pcm_hw_params_any(pcm_handle, hw_params));
+	ALSA_ERRCHECK(snd_pcm_hw_params_any(pcm_handle, hw_params));
 
 	if (g.s.doPositionalAudio()) {
 		iChannels = 1;
 		ALSA_ERRBAIL(snd_pcm_hw_params_get_channels_max(hw_params, &iChannels));
-		if (iChannels > 8) {
+		if (iChannels > 9) {
 			qWarning("ALSAAudioOutput: ALSA reports %d output channels. Clamping to 2.",iChannels);
 			iChannels = 2;
 		}
@@ -445,26 +448,39 @@ void ALSAAudioOutput::run() {
 
 	ALSA_ERRBAIL(snd_pcm_hw_params_set_access(pcm_handle, hw_params, SND_PCM_ACCESS_RW_INTERLEAVED));
 	ALSA_ERRBAIL(snd_pcm_hw_params_set_format(pcm_handle, hw_params, SND_PCM_FORMAT_S16));
-	ALSA_ERRBAIL(snd_pcm_hw_params_set_rate_near(pcm_handle, hw_params, &iMixerFreq, NULL));
 	ALSA_ERRBAIL(snd_pcm_hw_params_set_channels_near(pcm_handle, hw_params, &iChannels));
+	unsigned int rrate = SAMPLE_RATE;
+	ALSA_ERRBAIL(snd_pcm_hw_params_set_rate_near(pcm_handle, hw_params, &rrate, NULL));
 
-	unsigned int iOutputSize = (iFrameSize * iMixerFreq) / SAMPLE_RATE;
+	unsigned int iOutputSize = (iFrameSize * rrate) / SAMPLE_RATE;
 
 	snd_pcm_uframes_t period_size = iOutputSize;
 	snd_pcm_uframes_t buffer_size = iOutputSize * (g.s.iOutputDelay + 1);
 
-	ALSA_ERRBAIL(snd_pcm_hw_params_set_period_size_near(pcm_handle, hw_params, &period_size, NULL));
+	int dir = 1;
+	ALSA_ERRBAIL(snd_pcm_hw_params_set_period_size_near(pcm_handle, hw_params, &period_size, &dir));
 	ALSA_ERRBAIL(snd_pcm_hw_params_set_buffer_size_near(pcm_handle, hw_params, &buffer_size));
 
 	ALSA_ERRBAIL(snd_pcm_hw_params(pcm_handle, hw_params));
 	ALSA_ERRBAIL(snd_pcm_hw_params_current(pcm_handle, hw_params));
+	ALSA_ERRBAIL(snd_pcm_hw_params_get_period_size(hw_params, &period_size, &dir));
+	ALSA_ERRBAIL(snd_pcm_hw_params_get_buffer_size(hw_params, &buffer_size));
+
+	qWarning("ALSAAudioOutput: Actual buffer %d hz, %d channel %ld samples [%ld per period]",rrate,iChannels,buffer_size,period_size);
 
 	ALSA_ERRBAIL(snd_pcm_sw_params_current(pcm_handle, sw_params));
-	ALSA_ERRBAIL(snd_pcm_sw_params_set_start_threshold(pcm_handle, sw_params, 0));
 	ALSA_ERRBAIL(snd_pcm_sw_params_set_avail_min(pcm_handle, sw_params, period_size));
-
+	ALSA_ERRBAIL(snd_pcm_sw_params_set_start_threshold(pcm_handle, sw_params, buffer_size - period_size));
+	ALSA_ERRBAIL(snd_pcm_sw_params_set_stop_threshold(pcm_handle, sw_params, buffer_size));
 	ALSA_ERRBAIL(snd_pcm_sw_params(pcm_handle, sw_params));
 
+#ifdef ALSA_VERBOSE
+	snd_output_t *log;
+	snd_output_stdio_attach(&log, stderr,0);
+	if (pcm_handle)
+		snd_pcm_dump(pcm_handle, log);
+#endif
+
 	ALSA_ERRBAIL(snd_pcm_prepare(pcm_handle));
 
 	const unsigned int buffsize = static_cast<unsigned int>(period_size * iChannels);
@@ -492,10 +508,10 @@ void ALSAAudioOutput::run() {
 	const unsigned int chanmasks[32] = {
 		SPEAKER_FRONT_LEFT,
 		SPEAKER_FRONT_RIGHT,
-		SPEAKER_FRONT_CENTER,
-		SPEAKER_LOW_FREQUENCY,
 		SPEAKER_BACK_LEFT,
 		SPEAKER_BACK_RIGHT,
+		SPEAKER_FRONT_CENTER,
+		SPEAKER_LOW_FREQUENCY,
 		SPEAKER_SIDE_LEFT,
 		SPEAKER_SIDE_RIGHT,
 		SPEAKER_BACK_CENTER
@@ -503,7 +519,8 @@ void ALSAAudioOutput::run() {
 
 	ALSA_ERRBAIL(snd_pcm_hw_params_current(pcm_handle, hw_params));
 	ALSA_ERRBAIL(snd_pcm_hw_params_get_channels(hw_params, &iChannels));
-	ALSA_ERRBAIL(snd_pcm_hw_params_get_rate(hw_params, &iMixerFreq, NULL));
+	ALSA_ERRBAIL(snd_pcm_hw_params_get_rate(hw_params, &rrate, NULL));
+	iMixerFreq = rrate;
 	eSampleFormat = SampleShort;
 
 	qWarning("ALSAAudioOutput: Initializing %d channel, %d hz mixer", iChannels, iMixerFreq);
@@ -514,7 +531,7 @@ void ALSAAudioOutput::run() {
 
 	qml.unlock();
 
-	while (bRunning) {
+	while (bRunning && bOk) {
 		poll(fds, count, 20);
 		unsigned short revents;
 
@@ -522,26 +539,35 @@ void ALSAAudioOutput::run() {
 		if (revents & POLLERR) {
 			snd_pcm_prepare(pcm_handle);
 		} else if (revents & POLLOUT) {
-			while (snd_pcm_avail_update(pcm_handle) >= static_cast<int>(period_size)) {
+			snd_pcm_sframes_t avail;
+			ALSA_ERRCHECK(avail = snd_pcm_avail_update(pcm_handle));
+			while (avail >= static_cast<int>(period_size)) {
 				stillRun = mix(outbuff, static_cast<int>(period_size));
-				snd_pcm_sframes_t w = 0;
-				if (stillRun)
-					w=snd_pcm_writei(pcm_handle, outbuff, period_size);
-				else
+				if (stillRun) {
+					snd_pcm_sframes_t w = 0;
+					ALSA_ERRCHECK(w=snd_pcm_writei(pcm_handle, outbuff, period_size));
+					if (w < 0) {
+						avail = w;
+						break;
+					}
+				} else
 					break;
-				if (w == -EPIPE) {
-					qWarning("ALSAAudioOutput: %s", snd_strerror(static_cast<int>(w)));
-					snd_pcm_prepare(pcm_handle);
-					for (unsigned int i=0;i< buffer_size / period_size;i++)
-						snd_pcm_writei(pcm_handle, zerobuff, period_size);
-				}
+				ALSA_ERRCHECK(avail = snd_pcm_avail_update(pcm_handle));
+			}
+
+			if (avail == -EPIPE) {
+				snd_pcm_drain(pcm_handle);
+				ALSA_ERRCHECK(snd_pcm_prepare(pcm_handle));
+				for (unsigned int i=0;i< buffer_size / period_size;++i)
+					ALSA_ERRCHECK(snd_pcm_writei(pcm_handle, zerobuff, period_size));
 			}
 
 			if (! stillRun) {
 				snd_pcm_drain(pcm_handle);
 
-				while (bRunning && !mix(outbuff, period_size))
-					this->msleep(20);
+				while (bRunning && !mix(outbuff, period_size)) {
+					this->msleep(10);
+				}
 
 				if (! bRunning)
 					break;
