@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2009, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2011, Thorvald Natvig <thorvald@natvig.com>
 
    All rights reserved.
 
@@ -28,10 +28,11 @@
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
-#ifndef _ABOUT_H
-#define _ABOUT_H
+#ifndef ABOUT_H_
+#define ABOUT_H_
 
-#include "mumble_pch.hpp"
+#include <QtCore/QObject>
+#include <QtGui/QDialog>
 
 class AboutDialog : public QDialog {
 	private:
@@ -41,12 +42,4 @@ class AboutDialog : public QDialog {
 		AboutDialog(QWidget *parent);
 };
 
-class AboutSpeexDialog : public QDialog {
-	private:
-		Q_OBJECT
-		Q_DISABLE_COPY(AboutSpeexDialog)
-	public:
-		AboutSpeexDialog(QWidget *parent);
-};
-
 #endif
