@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2011, Thorvald Natvig <thorvald@natvig.com>
 
    All rights reserved.
 
@@ -28,8 +28,8 @@
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
-#ifndef _AUDIOCONFIGDIALOG_H
-#define _AUDIOCONFIGDIALOG_H
+#ifndef AUDIOCONFIGDIALOG_H_
+#define AUDIOCONFIGDIALOG_H_
 
 #include "ConfigDialog.h"
 
@@ -68,11 +68,12 @@ class AudioInputDialog : public ConfigWidget, public Ui::AudioInput {
 		void on_qsQuality_valueChanged(int v);
 		void on_qsAmp_valueChanged(int v);
 		void on_qsDoublePush_valueChanged(int v);
+		void on_qsPTTHold_valueChanged(int v);
 		void on_qsNoise_valueChanged(int v);
-		void on_qsIdle_valueChanged(int v);
 		void on_qcbTransmit_currentIndexChanged(int v);
 		void on_qcbSystem_currentIndexChanged(int);
 		void on_Tick_timeout();
+		void on_qcbIdleAction_currentIndexChanged(int v);
 };
 
 class AudioOutputDialog : public ConfigWidget, public Ui::AudioOutput {
