@@ -1,4 +1,4 @@
-/* Copyright (C) 2009, Mikkel Krautz <mikkel@krautz.dk>
+/* Copyright (C) 2009-2011, Mikkel Krautz <mikkel@krautz.dk>
 
    All rights reserved.
 
@@ -28,8 +28,12 @@
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
+#include "mumble_pch.hpp"
+
 #include "CrashReporter.h"
+
 #include "Global.h"
+#include "NetworkConfig.h"
 #include "OSInfo.h"
 
 CrashReporter::CrashReporter(QWidget *p) : QDialog(p) {
@@ -56,7 +60,7 @@ CrashReporter::CrashReporter(QWidget *p) : QDialog(p) {
 
 	qteDescription=new QTextEdit();
 	l->setBuddy(qteDescription);
-	l = new QLabel(tr("Please briefly describe what you were doing at the time of the crash"));
+	l = new QLabel(tr("Please describe briefly, in English, what you were doing at the time of the crash"));
 
 	vbl->addWidget(l);
 	vbl->addWidget(qteDescription, 1);
@@ -81,8 +85,7 @@ CrashReporter::CrashReporter(QWidget *p) : QDialog(p) {
 
 CrashReporter::~CrashReporter() {
 	g.qs->setValue(QLatin1String("crashemail"), qleEmail->text());
-	if (qnrReply)
-		delete qnrReply;
+	delete qnrReply;
 }
 
 void CrashReporter::uploadFinished() {
@@ -105,15 +108,7 @@ void CrashReporter::uploadProgress(qint64 sent, qint64 total) {
 
 void CrashReporter::run() {
 	QByteArray qbaDumpContents;
-#ifdef COMPAT_CLIENT
-#ifdef Q_OS_MAC
-	QFile qfCrashDump(QDir::homePath() + QLatin1String("/Library/Preferences/Mumble/mumble11x.dmp"));
-#else
-	QFile qfCrashDump(QDesktopServices::storageLocation(QDesktopServices::DataLocation) + QLatin1String("/mumble11x.dmp"));
-#endif
-#else
 	QFile qfCrashDump(g.qdBasePath.filePath(QLatin1String("mumble.dmp")));
-#endif
 	if (! qfCrashDump.exists())
 		return;
 
@@ -136,17 +131,13 @@ void CrashReporter::run() {
 	QDateTime qdtModification = qfiDump.lastModified();
 
 	/* Find the real crash report. */
-	QDir qdCrashReports = QDir::home().absolutePath() + QLatin1String("/Library/Logs/DiagnosticReports/");
+	QDir qdCrashReports(QDir::home().absolutePath() + QLatin1String("/Library/Logs/DiagnosticReports/"));
 	if (! qdCrashReports.exists()) {
 		qdCrashReports = QDir::home().absolutePath() + QLatin1String("/Library/Logs/CrashReporter/");
 	}
 
 	QStringList qslFilters;
-#ifdef COMPAT_CLIENT
-	qslFilters << QString::fromLatin1("Mumble11x_*.crash");
-#else
 	qslFilters << QString::fromLatin1("Mumble_*.crash");
-#endif
 	qdCrashReports.setNameFilters(qslFilters);
 	qdCrashReports.setSorting(QDir::Time);
 	QFileInfoList qfilEntries = qdCrashReports.entryInfoList();
@@ -166,6 +157,41 @@ void CrashReporter::run() {
 	}
 #endif
 
+	QString details;
+#ifdef Q_OS_WIN
+	{
+		QTemporaryFile qtf;
+		if (qtf.open()) {
+			qtf.close();
+
+			QProcess qp;
+			QStringList qsl;
+
+			qsl << QLatin1String("/t");
+			qsl << qtf.fileName();
+
+			QString app = QLatin1String("dxdiag.exe");
+			wchar_t *sr = NULL;
+			size_t srsize = 0;
+			if (_wdupenv_s(&sr, &srsize, L"SystemRoot") == 0) {
+				app = QDir::fromNativeSeparators(QString::fromWCharArray(sr)) + QLatin1String("/System32/dxdiag.exe");
+				free(sr);
+			}
+
+			qp.start(app, qsl);
+			if (qp.waitForFinished(30000)) {
+				if (qtf.open()) {
+					QByteArray qba = qtf.readAll();
+					details = QString::fromLocal8Bit(qba);
+				}
+			} else {
+				details = QLatin1String("Failed to run dxdiag");
+			}
+			qp.kill();
+		}
+	}
+#endif
+
 	if (qbaDumpContents.isEmpty()) {
 		qWarning("CrashReporter: Empty crash dump file, not reporting.");
 		return;
@@ -183,15 +209,17 @@ void CrashReporter::run() {
 		QString ver = QString::fromLatin1("--%1\r\nContent-Disposition: form-data; name=\"ver\"\r\nContent-Transfer-Encoding: 8bit\r\n\r\n%2 %3\r\n").arg(boundary, QLatin1String(MUMTEXT(MUMBLE_VERSION_STRING)), QLatin1String(MUMBLE_RELEASE));
 		QString email = QString::fromLatin1("--%1\r\nContent-Disposition: form-data; name=\"email\"\r\nContent-Transfer-Encoding: 8bit\r\n\r\n%2\r\n").arg(boundary, qleEmail->text());
 		QString descr = QString::fromLatin1("--%1\r\nContent-Disposition: form-data; name=\"desc\"\r\nContent-Transfer-Encoding: 8bit\r\n\r\n%2\r\n").arg(boundary, qteDescription->toPlainText());
+		QString det = QString::fromLatin1("--%1\r\nContent-Disposition: form-data; name=\"details\"\r\nContent-Transfer-Encoding: 8bit\r\n\r\n%2\r\n").arg(boundary, details);
 		QString head = QString::fromLatin1("--%1\r\nContent-Disposition: form-data; name=\"dump\"; filename=\"mumble.dmp\"\r\nContent-Type: binary/octet-stream\r\n\r\n").arg(boundary);
 		QString end = QString::fromLatin1("\r\n--%1--\r\n").arg(boundary);
 
-		QByteArray post = os.toUtf8() + ver.toUtf8() + email.toUtf8() + descr.toUtf8() + head.toUtf8() + qbaDumpContents + end.toUtf8();
+		QByteArray post = os.toUtf8() + ver.toUtf8() + email.toUtf8() + descr.toUtf8() + det.toUtf8() + head.toUtf8() + qbaDumpContents + end.toUtf8();
 
 		QUrl url(QLatin1String("https://mumble.hive.no/crashreport.php"));
 		QNetworkRequest req(url);
 		req.setHeader(QNetworkRequest::ContentTypeHeader, QString::fromLatin1("multipart/form-data; boundary=%1").arg(boundary));
 		req.setHeader(QNetworkRequest::ContentLengthHeader, QString::number(post.size()));
+		Network::prepareRequest(req);
 		qnrReply = g.nam->post(req, post);
 		connect(qnrReply, SIGNAL(finished()), this, SLOT(uploadFinished()));
 		connect(qnrReply, SIGNAL(uploadProgress(qint64, qint64)), this, SLOT(uploadProgress(qint64, qint64)));
