@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2011, Thorvald Natvig <thorvald@natvig.com>
 
    All rights reserved.
 
@@ -28,7 +28,10 @@
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
+#include "mumble_pch.hpp"
+
 #include "DirectSound.h"
+
 #include "MainWindow.h"
 #include "Plugins.h"
 #include "User.h"
@@ -40,10 +43,7 @@
 // #define MY_DEFERRED DS3D_DEFERRED
 #define MY_DEFERRED DS3D_IMMEDIATE
 
-
 #define NBLOCKS 50
-#define MAX(a,b)        ( (a) > (b) ? (a) : (b) )
-#define MIN(a,b)        ( (a) < (b) ? (a) : (b) )
 
 class DXAudioOutputRegistrar : public AudioOutputRegistrar {
 	public:
@@ -68,6 +68,7 @@ class DirectSoundInit : public DeferInit {
 		DXAudioInputRegistrar *airReg;
 		DXAudioOutputRegistrar *aorReg;
 	public:
+		DirectSoundInit() : airReg(NULL), aorReg(NULL) {}
 		void initialize();
 		void destroy();
 };
@@ -102,10 +103,8 @@ void DirectSoundInit::initialize() {
 }
 
 void DirectSoundInit::destroy() {
-	if (airReg)
-		delete airReg;
-	if (aorReg)
-		delete aorReg;
+	delete airReg;
+	delete aorReg;
 }
 
 
@@ -206,7 +205,6 @@ bool DXAudioInputRegistrar::canEcho(const QString &) const {
 }
 
 DXAudioOutput::DXAudioOutput() {
-	bRunning = true;
 }
 
 DXAudioOutput::~DXAudioOutput() {
@@ -227,10 +225,6 @@ void DXAudioOutput::run() {
 	LPDIRECTSOUNDBUFFER       pDSBOutput = NULL;
 	LPDIRECTSOUNDNOTIFY8       pDSNotify = NULL;
 
-	DWORD	dwBufferSize;
-	DWORD	dwLastWritePos;
-	DWORD	dwLastPlayPos;
-	DWORD	dwTotalPlayPos;
 	int iLastwriteblock;
 	LPVOID aptr1, aptr2;
 	DWORD nbytes1, nbytes2;
@@ -262,13 +256,13 @@ void DXAudioOutput::run() {
 	}
 
 	if (! pDS && FAILED(hr = DirectSoundCreate8(&DSDEVID_DefaultVoicePlayback, &pDS, NULL))) {
-		qWarning("DXAudioOutput: DirectSoundCreate");
+		qWarning("DXAudioOutput: DirectSoundCreate failed: hr=0x%08lx", hr);
 		goto cleanup;
 	} else if (FAILED(hr = pDS->SetCooperativeLevel(g.mw->winId(), DSSCL_PRIORITY))) {
-		qWarning("DXAudioOutput: SetCooperativeLevel");
+		qWarning("DXAudioOutput: SetCooperativeLevel failed: hr=0x%08lx", hr);
 		goto cleanup;
 	} else if (FAILED(hr = pDS->CreateSoundBuffer(&dsbdesc, &pDSBPrimary, NULL))) {
-		qWarning("DXAudioOutput: CreateSoundBuffer (Primary) : 0x%08lx", hr);
+		qWarning("DXAudioOutput: CreateSoundBuffer (Primary) failed: hr=0x%08lx", hr);
 		goto cleanup;
 	}
 
@@ -310,7 +304,7 @@ void DXAudioOutput::run() {
 	}
 
 	if (! g.s.doPositionalAudio())
-		dwMask = KSAUDIO_SPEAKER_MONO;
+		dwMask = KSAUDIO_SPEAKER_STEREO;
 
 	for (int i=0;i<32;i++) {
 		if (dwMask & (1 << i)) {
@@ -329,18 +323,18 @@ void DXAudioOutput::run() {
 
 	ZeroMemory(&wfx, sizeof(wfx));
 	wfx.Format.wFormatTag = WAVE_FORMAT_PCM;
-	wfx.Format.nChannels = qMax(ns, 1);
+	wfx.Format.nChannels = qMax(ns, 2);
 	wfx.Format.nSamplesPerSec = SAMPLE_RATE;
 	wfx.Format.nBlockAlign = sizeof(short) * wfx.Format.nChannels;
 	wfx.Format.nAvgBytesPerSec = wfx.Format.nSamplesPerSec * wfx.Format.nBlockAlign;
 	wfx.Format.wBitsPerSample = 16;
 
 	if (FAILED(hr = pDSBPrimary->SetFormat(reinterpret_cast<WAVEFORMATEX *>(&wfx)))) {
-		qWarning("DXAudioOutput: SetFormat");
+		qWarning("DXAudioOutput: SetFormat failed: hr=0x%08lx", hr);
 		goto cleanup;
 	}
 	if (FAILED(hr = pDSBPrimary->GetFormat(reinterpret_cast<WAVEFORMATEX *>(&wfxSet), sizeof(wfxSet), NULL))) {
-		qWarning("DXAudioOutput: GetFormat");
+		qWarning("DXAudioOutput: GetFormat failed: hr=0x%08lx", hr);
 		goto cleanup;
 	}
 
@@ -348,7 +342,7 @@ void DXAudioOutput::run() {
 
 	ZeroMemory(&wfx, sizeof(wfx));
 	wfx.Format.wFormatTag = WAVE_FORMAT_EXTENSIBLE;
-	wfx.Format.nChannels = ns;
+	wfx.Format.nChannels = qMax(ns, 2);
 	wfx.Format.nSamplesPerSec = SAMPLE_RATE;
 	wfx.Format.nBlockAlign = sizeof(short) * wfx.Format.nChannels;
 	wfx.Format.nAvgBytesPerSec = wfx.Format.nSamplesPerSec * wfx.Format.nBlockAlign;
@@ -366,13 +360,13 @@ void DXAudioOutput::run() {
 	dsbdesc.lpwfxFormat     = reinterpret_cast<WAVEFORMATEX *>(&wfx);
 
 	if (FAILED(hr = pDS->CreateSoundBuffer(&dsbdesc, &pDSBOutput, NULL))) {
-		qWarning("DXAudioOutputUser: CreateSoundBuffer (Secondary): 0x%08lx", hr);
+		qWarning("DXAudioOutputUser: CreateSoundBuffer (Secondary) failed: hr=0x%08lx", hr);
 		goto cleanup;
 	}
 
 
 	if (FAILED(hr = pDSBOutput->QueryInterface(IID_IDirectSoundNotify, reinterpret_cast<void **>(&pDSNotify)))) {
-		qWarning("DXAudioOutputUser: QueryInterface (Notify)");
+		qWarning("DXAudioOutputUser: QueryInterface (Notify) failed: hr=0x%08lx", hr);
 		goto cleanup;
 	}
 
@@ -384,37 +378,32 @@ void DXAudioOutput::run() {
 	initializeMixer(chanmasks, bHead);
 
 	if (FAILED(hr = pDSBOutput->Lock(0, 0, &aptr1, &nbytes1, &aptr2, &nbytes2, DSBLOCK_ENTIREBUFFER))) {
-		qWarning("DXAudioOutputUser: Initial Lock");
+		qWarning("DXAudioOutputUser: Initial Lock failed: hr=0x%08lx", hr);
 		goto cleanup;
 	}
 
-	dwBufferSize = nbytes1 + nbytes2;
 	if (aptr1)
 		ZeroMemory(aptr1, nbytes1);
 	if (aptr2)
 		ZeroMemory(aptr2, nbytes2);
 
 	if (FAILED(hr = pDSBOutput->Unlock(aptr1, nbytes1, aptr2, nbytes2))) {
-		qWarning("DXAudioOutputUser: Initial Unlock");
+		qWarning("DXAudioOutputUser: Initial Unlock failed: hr=0x%08lx", hr);
 		goto cleanup;
 	}
 
 	if (FAILED(hr = pDSBOutput->Play(0, 0, DSBPLAY_LOOPING))) {
-		qWarning("DXAudioOutputUser: Play");
+		qWarning("DXAudioOutputUser: Play failed: hr=0x%08lx", hr);
 		goto cleanup;
 	}
 
-	dwLastWritePos = 0;
-	dwLastPlayPos = 0;
-	dwTotalPlayPos = 0;
-
 	iLastwriteblock = (NBLOCKS - 1 + g.s.iOutputDelay) % NBLOCKS;
 
 	bOk = true;
 
 	while (bRunning && ! FAILED(hr)) {
 		if (FAILED(hr = pDSBOutput->GetCurrentPosition(&dwPlayPosition, &dwWritePosition))) {
-			qWarning("DXAudioOutputUser: GetCurrentPosition");
+			qWarning("DXAudioOutputUser: GetCurrentPosition failed: hr=0x%08lx", hr);
 			break;
 		}
 
@@ -425,7 +414,7 @@ void DXAudioOutput::run() {
 			iLastwriteblock = block;
 
 			if (FAILED(hr = pDSBOutput->Lock(block * iByteSize, iByteSize, &aptr1, &nbytes1, &aptr2, &nbytes2, 0))) {
-				qWarning("DXAudioOutput: Lock block %u (%d bytes)",block, iByteSize);
+				qWarning("DXAudioOutput: Lock block %u (%d bytes) failed: hr=0x%08lx",block, iByteSize, hr);
 				break;
 			}
 			if (aptr2 || nbytes2) {
@@ -440,12 +429,12 @@ void DXAudioOutput::run() {
 				ZeroMemory(aptr1, iByteSize);
 
 			if (FAILED(hr = pDSBOutput->Unlock(aptr1, nbytes1, aptr2, nbytes2))) {
-				qWarning("DXAudioOutput: Unlock %p(%lu) %p(%lu)",aptr1,nbytes1,aptr2,nbytes2);
+				qWarning("DXAudioOutput: Unlock %p(%lu) %p(%lu) failed: hr=0x%08lx",aptr1,nbytes1,aptr2,nbytes2,hr);
 				break;
 			}
 
 			if (FAILED(hr = pDSBOutput->GetCurrentPosition(&dwPlayPosition, &dwWritePosition))) {
-				qWarning("DXAudioOutputUser: GetCurrentPosition");
+				qWarning("DXAudioOutputUser: GetCurrentPosition failed: hr=0x%08lx", hr);
 				break;
 			}
 
@@ -480,7 +469,6 @@ cleanup:
 #define NBUFFBLOCKS 50
 
 DXAudioInput::DXAudioInput() {
-	bRunning = true;
 }
 
 DXAudioInput::~DXAudioInput() {
@@ -495,8 +483,6 @@ void DXAudioInput::run() {
 
 	DWORD dwBufferSize;
 	bool bOk;
-	DWORD dwReadyBytes;
-	DWORD dwLastReadPos = 0;
 	DWORD dwReadPosition;
 	DWORD dwCapturePosition;
 
@@ -514,9 +500,6 @@ void DXAudioInput::run() {
 	bOk = false;
 
 	bool failed = false;
-	float safety = 2.0f;
-	bool didsleep = false;
-	bool firstsleep = false;
 
 	Timer t;
 
@@ -544,11 +527,11 @@ void DXAudioInput::run() {
 	}
 
 	if (! pDSCapture && FAILED(hr = DirectSoundCaptureCreate8(&DSDEVID_DefaultVoiceCapture, &pDSCapture, NULL)))
-		qWarning("DXAudioInput: DirectSoundCaptureCreate");
+		qWarning("DXAudioInput: DirectSoundCaptureCreate failed: hr=0x%08lx", hr);
 	else if (FAILED(hr = pDSCapture->CreateCaptureBuffer(&dscbd, &pDSCaptureBuffer, NULL)))
-		qWarning("DXAudioInput: CreateCaptureBuffer");
+		qWarning("DXAudioInput: CreateCaptureBuffer failed: hr=0x%08lx", hr);
 	else if (FAILED(hr = pDSCaptureBuffer->QueryInterface(IID_IDirectSoundNotify, reinterpret_cast<void **>(&pDSNotify))))
-		qWarning("DXAudioInput: QueryInterface (Notify)");
+		qWarning("DXAudioInput: QueryInterface (Notify) failed: hr=0x%08lx", hr);
 	else
 		bOk = true;
 
@@ -563,15 +546,19 @@ void DXAudioInput::run() {
 		goto cleanup;
 
 	if (FAILED(hr = pDSCaptureBuffer->Start(DSCBSTART_LOOPING))) {
-		qWarning("DXAudioInput: Start failed");
+		qWarning("DXAudioInput: Start failed: hr=0x%08lx", hr);
 	} else {
+		DWORD dwReadyBytes = 0;
+		DWORD dwLastReadPos = 0;
+		float safety = 2.0f;
+
 		while (bRunning) {
-			firstsleep = true;
-			didsleep = false;
+			bool firstsleep = true;
+			bool didsleep = false;
 
 			do {
 				if (FAILED(hr = pDSCaptureBuffer->GetCurrentPosition(&dwCapturePosition, &dwReadPosition))) {
-					qWarning("DXAudioInput: GetCurrentPosition");
+					qWarning("DXAudioInput: GetCurrentPosition failed: hr=0x%08lx", hr);
 					bRunning = false;
 					break;
 				}
@@ -603,7 +590,7 @@ void DXAudioInput::run() {
 				dwLastReadPos = dwReadPosition;
 			} else if (bRunning) {
 				if (FAILED(hr = pDSCaptureBuffer->Lock(dwLastReadPos, sizeof(short) * iFrameSize, &aptr1, &nbytes1, &aptr2, &nbytes2, 0))) {
-					qWarning("DXAudioInput: Lock from %ld (%d bytes)",dwLastReadPos, sizeof(short) * iFrameSize);
+					qWarning("DXAudioInput: Lock from %ld (%d bytes) failed: hr=0x%08lx",dwLastReadPos, sizeof(short) * iFrameSize, hr);
 					bRunning = false;
 					break;
 				}
@@ -615,7 +602,7 @@ void DXAudioInput::run() {
 					CopyMemory(psMic+nbytes1/2, aptr2, nbytes2);
 
 				if (FAILED(hr = pDSCaptureBuffer->Unlock(aptr1, nbytes1, aptr2, nbytes2))) {
-					qWarning("DXAudioInput: Unlock");
+					qWarning("DXAudioInput: Unlock failed: hr=0x%08lx", hr);
 					bRunning = false;
 					break;
 				}
