@@ -1,5 +1,5 @@
-/* Copyright (C) 2005-2009, Thorvald Natvig <thorvald@natvig.com>
-   Copyright (C) 2008-2009, Mikkel Krautz <mikkel@krautz.dk>
+/* Copyright (C) 2005-2011, Thorvald Natvig <thorvald@natvig.com>
+   Copyright (C) 2008-2011, Mikkel Krautz <mikkel@krautz.dk>
 
    All rights reserved.
 
@@ -29,6 +29,8 @@
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
+#include "mumble_pch.hpp"
+
 #include <G15LCDEngine_unix.h>
 
 static LCDEngine *G15LCDEngineNew() {
@@ -48,9 +50,6 @@ G15LCDEngineUnix::G15LCDEngineUnix() {
 }
 
 G15LCDEngineUnix::~G15LCDEngineUnix() {
-	foreach(LCDDevice *d, qlDevices) {
-		delete d;
-	}
 }
 
 QList<LCDDevice *> G15LCDEngineUnix::devices() const {
