@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2009, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2011, Thorvald Natvig <thorvald@natvig.com>
 
    All rights reserved.
 
@@ -28,28 +28,17 @@
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
-#include "GlobalShortcut_unix.h"
-#include "Global.h"
-#include <QX11Info>
-#include <X11/X.h>
-#include <X11/Xlib.h>
-#ifdef USE_XEVIE
-#include <X11/extensions/Xevie.h>
-#endif
+#include "mumble_pch.hpp"
 
-#ifdef Q_OS_LINUX
-#include <linux/input.h>
-#include <fcntl.h>
-#endif
+#include "GlobalShortcut_unix.h"
 
 GlobalShortcutEngine *GlobalShortcutEngine::platformInit() {
 	return new GlobalShortcutX();
 }
 
 GlobalShortcutX::GlobalShortcutX() {
-	int min, maj;
-	bRunning=false;
-	bXevie = false;
+	iXIopcode =  -1;
+	bRunning = false;
 
 	display = NULL;
 
@@ -65,7 +54,7 @@ GlobalShortcutX::GlobalShortcutX() {
 		qmInputDevices.clear();
 
 		delete fsw;
-		qWarning("GlobalShortcutX: Unable to open any keyboard input devices under /dev/input, falling back to XEVIE");
+		qWarning("GlobalShortcutX: Unable to open any keyboard input devices under /dev/input, falling back to XInput");
 	} else {
 		return;
 	}
@@ -78,31 +67,48 @@ GlobalShortcutX::GlobalShortcutX() {
 		return;
 	}
 
-	maj = 1;
-	min = 1;
+	for (int i=0; i < ScreenCount(display); ++i)
+		qsRootWindows.insert(RootWindow(display, i));
 
-#ifdef USE_XEVIE
-	if (! XevieQueryVersion(display, &maj, &min)) {
-		qWarning("GlobalShortcutX: XEVIE extension not found. Enable it in xorg.conf");
-	} else {
-		qWarning("GlobalShortcutX: XEVIE %d.%d", maj, min);
+#ifndef NO_XINPUT2
+	int evt, error;
 
-#ifdef QT_NO_DEBUG
-		if (! XevieStart(display)) {
-			qWarning("GlobalShortcutX: Another client is already using XEVIE");
-		} else {
-			XevieSelectInput(display, KeyPressMask | KeyReleaseMask | ButtonPressMask | ButtonReleaseMask);
-			bXevie = true;
+	if (XQueryExtension(display, "XInputExtension", &iXIopcode, &evt, &error)) {
+		int major = 2;
+		int minor = 0;
+		int rc = XIQueryVersion(display, &major, &minor);
+		if (rc != BadRequest) {
+			qWarning("GlobalShortcutX: Using XI2 %d.%d", major, minor);
+
+			queryXIMasterList();
+
+			XIEventMask evmask;
+			unsigned char mask[(XI_LASTEVENT + 7)/8];
+
+			memset(&evmask, 0, sizeof(evmask));
+			memset(mask, 0, sizeof(mask));
+
+			XISetMask(mask, XI_RawButtonPress);
+			XISetMask(mask, XI_RawButtonRelease);
+			XISetMask(mask, XI_RawKeyPress);
+			XISetMask(mask, XI_RawKeyRelease);
+			XISetMask(mask, XI_HierarchyChanged);
+
+			evmask.deviceid = XIAllDevices;
+			evmask.mask_len = sizeof(mask);
+			evmask.mask = mask;
+
+			foreach(Window w, qsRootWindows)
+				XISelectEvents(display, w, &evmask, 1);
+			XFlush(display);
+
+			connect(new QSocketNotifier(ConnectionNumber(display), QSocketNotifier::Read, this), SIGNAL(activated(int)), this, SLOT(displayReadyRead(int)));
+
+			return;
 		}
-#endif
 	}
-#else
-	qWarning("GlobalShortcutX: Not compiled with XEVIE support.");
 #endif
-	if (! bXevie) {
-		qWarning("GlobalShortcutX: No XEVIE support, falling back to polled input. This wastes a lot of CPU resources, so please enable one of the other methods.");
-	}
-
+	qWarning("GlobalShortcutX: No XInput support, falling back to polled input. This wastes a lot of CPU resources, so please enable one of the other methods.");
 	bRunning=true;
 	start(QThread::TimeCriticalPriority);
 }
@@ -110,105 +116,118 @@ GlobalShortcutX::GlobalShortcutX() {
 GlobalShortcutX::~GlobalShortcutX() {
 	bRunning = false;
 	wait();
+
+	if (display)
+		XCloseDisplay(display);
 }
 
-bool GlobalShortcutX::canSuppress() {
-	return bXevie;
-}
-
+// Tight loop polling
 void GlobalShortcutX::run() {
-	fd_set in_fds;
-	XEvent evt;
-	struct timeval tv;
+	Window root = XDefaultRootWindow(display);
+	Window root_ret, child_ret;
+	int root_x, root_y;
+	int win_x, win_y;
+	unsigned int mask[2];
+	int idx = 0;
+	int next = 0;
+	char keys[2][32];
 
-	if (bXevie) {
-#ifdef USE_XEVIE
-		while (bRunning) {
-			if (bNeedRemap)
-				remap();
-			FD_ZERO(&in_fds);
-			FD_SET(ConnectionNumber(display), &in_fds);
-			tv.tv_sec = 0;
-			tv.tv_usec = 100000;
-			if (select(ConnectionNumber(display)+1, &in_fds, NULL, NULL, &tv)) {
-				while (XPending(display)) {
-					bool suppress = false;
-					XNextEvent(display, &evt);
+	memset(keys[0], 0, 32);
+	memset(keys[1], 0, 32);
+	mask[0] = mask[1] = 0;
 
-					switch (evt.type) {
-						case KeyPress:
-						case KeyRelease:
-						case ButtonPress:
-						case ButtonRelease: {
-								bool down = (evt.type == KeyPress || evt.type == ButtonPress);
-								int evtcode;
-								if (evt.type == KeyPress || evt.type == KeyRelease)
-									evtcode = evt.xkey.keycode;
-								else
-									evtcode = 0x118 + evt.xbutton.button;
+	while (bRunning) {
+		if (bNeedRemap)
+			remap();
 
-								suppress = handleButton(evtcode, down);
-							}
-							break;
-						default:
-							qWarning("GlobalShortcutX: EVT %x", evt.type);
-					}
-					if (! suppress)
-						XevieSendEvent(display, &evt, XEVIE_UNMODIFIED);
+		msleep(10);
+
+		idx = next;
+		next = idx ^ 1;
+		if (XQueryPointer(display, root, &root_ret, &child_ret, &root_x, &root_y, &win_x, &win_y, &mask[next]) && XQueryKeymap(display, keys[next])) {
+			for (int i=0;i<256;++i) {
+				int index = i / 8;
+				int keymask = 1 << (i % 8);
+				bool oldstate = (keys[idx][index] & keymask) != 0;
+				bool newstate = (keys[next][index] & keymask) != 0;
+				if (oldstate != newstate) {
+					handleButton(i, newstate);
 				}
 			}
-		}
-		XevieEnd(display);
-#endif
-	} else {
-		Window root = XDefaultRootWindow(display);
-		Window root_ret, child_ret;
-		int root_x, root_y;
-		int win_x, win_y;
-		unsigned int mask[2];
-		int idx = 0;
-		int next = 0;
-		char keys[2][32];
-
-		memset(keys[0], 0, 32);
-		memset(keys[1], 0, 32);
-		mask[0] = mask[1] = 0;
-
-		while (bRunning) {
-			if (bNeedRemap)
-				remap();
-
-			msleep(10);
-
-			idx = next;
-			next = idx ^ 1;
-			if (! XQueryPointer(display, root, &root_ret, &child_ret, &root_x, &root_y, &win_x, &win_y, &mask[next]) ||
-			        ! XQueryKeymap(display, keys[next])) {
-				qWarning("GlobalShortcutX: Lost connection");
-				bRunning = false;
-			} else {
-				for (int i=0;i<256;++i) {
-					int index = i / 8;
-					int keymask = 1 << (i % 8);
-					bool oldstate = (keys[idx][index] & keymask) != 0;
-					bool newstate = (keys[next][index] & keymask) != 0;
-					if (oldstate != newstate) {
-						handleButton(i, newstate);
-					}
-				}
-				for (int i=8;i<=12;++i) {
-					bool oldstate = (mask[idx] & (1 << i)) != 0;
-					bool newstate = (mask[next] & (1 << i)) != 0;
-					if (oldstate != newstate) {
-						handleButton(0x110 + i, newstate);
-					}
+			for (int i=8;i<=12;++i) {
+				bool oldstate = (mask[idx] & (1 << i)) != 0;
+				bool newstate = (mask[next] & (1 << i)) != 0;
+				if (oldstate != newstate) {
+					handleButton(0x110 + i, newstate);
 				}
 			}
 		}
 	}
-	XCloseDisplay(display);
 }
 
+// Find XI2 master devices so they can be ignored.
+void GlobalShortcutX::queryXIMasterList() {
+#ifndef NO_XINPUT2
+	XIDeviceInfo *info, *dev;
+	int ndevices;
+
+	qsMasterDevices.clear();
+
+	dev = info = XIQueryDevice(display, XIAllDevices, &ndevices);
+	for (int i=0;i<ndevices;++i) {
+		switch (dev->use) {
+			case XIMasterPointer:
+			case XIMasterKeyboard:
+				qsMasterDevices.insert(dev->deviceid);
+				break;
+			default:
+				break;
+		}
+
+		++dev;
+	}
+	XIFreeDeviceInfo(info);
+#endif
+}
+
+// XInput2 event is ready on socketnotifier.
+void GlobalShortcutX::displayReadyRead(int) {
+#ifndef NO_XINPUT2
+	XEvent evt;
+
+	if (bNeedRemap)
+		remap();
+
+	while (XPending(display)) {
+		XNextEvent(display, &evt);
+		XGenericEventCookie *cookie = & evt.xcookie;
+
+		if ((cookie->type != GenericEvent) || (cookie->extension != iXIopcode) || !XGetEventData(display, cookie))
+			continue;
+
+		XIDeviceEvent *xide = reinterpret_cast<XIDeviceEvent *>(cookie->data);
+
+		switch (cookie->evtype) {
+			case XI_RawKeyPress:
+			case XI_RawKeyRelease:
+				if (! qsMasterDevices.contains(xide->deviceid))
+					handleButton(xide->detail, cookie->evtype == XI_RawKeyPress);
+				break;
+			case XI_RawButtonPress:
+			case XI_RawButtonRelease:
+				if (! qsMasterDevices.contains(xide->deviceid))
+					handleButton(xide->detail + 0x117, cookie->evtype == XI_RawButtonPress);
+				break;
+			case XI_HierarchyChanged:
+				queryXIMasterList();
+		}
+
+		XFreeEventData(display, cookie);
+	}
+#endif
+}
+
+// One of the raw /dev/input devices has ready input
 void GlobalShortcutX::inputReadyRead(int) {
 #ifdef Q_OS_LINUX
 	struct input_event ev;
@@ -256,6 +275,7 @@ void GlobalShortcutX::inputReadyRead(int) {
 
 #define test_bit(bit, array)    (array[bit/8] & (1<<(bit%8)))
 
+// The /dev/input directory changed
 void GlobalShortcutX::directoryChanged(const QString &dir) {
 #ifdef Q_OS_LINUX
 	QDir d(dir, QLatin1String("event*"), 0, QDir::System);
@@ -309,7 +329,7 @@ QString GlobalShortcutX::buttonName(const QVariant &v) {
 			return QLatin1String("0x")+QString::number(key,16);
 		} else {
 			const char *str=XKeysymToString(ks);
-			if (strlen(str) == 0) {
+			if (*str == '\0') {
 				return QLatin1String("KS0x")+QString::number(ks, 16);
 			} else {
 				return QLatin1String(str);
