@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2009, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2011, Thorvald Natvig <thorvald@natvig.com>
 
    All rights reserved.
 
@@ -35,7 +35,9 @@
 #include <QX11Info>
 #include <X11/X.h>
 #include <X11/Xlib.h>
-#include <X11/extensions/XInput.h>
+#ifndef NO_XINPUT2
+#include <X11/extensions/XInput2.h>
+#endif
 #include <X11/Xutil.h>
 #ifdef Q_OS_LINUX
 #include <linux/input.h>
@@ -50,20 +52,20 @@ class GlobalShortcutX : public GlobalShortcutEngine {
 		Q_DISABLE_COPY(GlobalShortcutX)
 	public:
 		Display *display;
+		QSet<Window> qsRootWindows;
+		int iXIopcode;
+		QSet<int> qsMasterDevices;
+
 		volatile bool bRunning;
-		bool bXInput;
 		QSet<QString> qsKeyboards;
 		QMap<QString, QFile *> qmInputDevices;
-		QMap<XID, XDevice *> qmXDevices;
 
 		GlobalShortcutX();
 		~GlobalShortcutX();
 		void run();
 		QString buttonName(const QVariant &);
 
-		int iKeyPress, iKeyRelease, iButtonPress, iButtonRelease;
-
-		void initXInput();
+		void queryXIMasterList();
 	public slots:
 		void displayReadyRead(int);
 		void inputReadyRead(int);
