@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2009, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2011, Thorvald Natvig <thorvald@natvig.com>
 
    All rights reserved.
 
@@ -29,6 +29,7 @@
 */
 
 #include "GlobalShortcut.h"
+#include "Timer.h"
 
 #define DIRECTINPUT_VERSION 0x0800
 #include <dinput.h>
@@ -59,14 +60,19 @@ class GlobalShortcutWin : public GlobalShortcutEngine {
 		Q_OBJECT
 		Q_DISABLE_COPY(GlobalShortcutWin)
 	public:
+		BYTE ucKeyState[256];
 		LPDIRECTINPUT8 pDI;
 		QHash<GUID, InputDevice *> qhInputDevices;
 		HHOOK hhMouse, hhKeyboard;
+		unsigned int uiHardwareDevices;
+		Timer tDoubleClick;
+		bool bHook;
 		static BOOL CALLBACK EnumSuitableDevicesCB(LPCDIDEVICEINSTANCE, LPDIRECTINPUTDEVICE8, DWORD, DWORD, LPVOID);
 		static BOOL CALLBACK EnumDevicesCB(LPCDIDEVICEINSTANCE, LPVOID);
 		static BOOL CALLBACK EnumDeviceObjectsCallback(LPCDIDEVICEOBJECTINSTANCE lpddoi, LPVOID pvRef);
 		static LRESULT CALLBACK HookKeyboard(int, WPARAM, LPARAM);
 		static LRESULT CALLBACK HookMouse(int, WPARAM, LPARAM);
+
 		virtual bool canSuppress();
 		void run();
 	public slots:
@@ -76,6 +82,8 @@ class GlobalShortcutWin : public GlobalShortcutEngine {
 		~GlobalShortcutWin();
 		void unacquire();
 		QString buttonName(const QVariant &);
+
+		virtual void prepareInput();
 };
 
 uint qHash(const GUID &);
