@@ -1,5 +1,5 @@
-/* Copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
-   Copyright (C) 2008-2009, Mikkel Krautz <mikkel@krautz.dk>
+/* Copyright (C) 2005-2011, Thorvald Natvig <thorvald@natvig.com>
+   Copyright (C) 2008-2011, Mikkel Krautz <mikkel@krautz.dk>
 
    All rights reserved.
 
@@ -29,7 +29,10 @@
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
+#include "mumble_pch.hpp"
+
 #include "LCD.h"
+
 #include "ClientUser.h"
 #include "Channel.h"
 #include "Global.h"
@@ -90,8 +93,7 @@ void LCDDeviceManager::destroy() {
 	foreach(LCDEngine *e, qlEngines) {
 		delete e;
 	}
-	if (crLCD)
-		delete crLCD;
+	delete crLCD;
 }
 
 static LCDDeviceManager devmgr;
@@ -319,7 +321,7 @@ void LCD::updateUserView() {
 		foreach(Channel *c, home->allLinks()) {
 			foreach(User *p, c->qlUsers) {
 				ClientUser *u = static_cast<ClientUser *>(p);
-				bool bTalk = (u->tsState != ClientUser::TalkingOff);
+				bool bTalk = (u->tsState != Settings::Passive);
 				if (bTalk) {
 					speaking.insert(p->uiSession, Timer());
 				} else if (qmSpeaking.contains(p->uiSession)) {
