@@ -1,5 +1,5 @@
-/* Copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
-   Copyright (C) 2009, Stefan Hacker <dd0t@users.sourceforge.net>
+/* Copyright (C) 2005-2011, Thorvald Natvig <thorvald@natvig.com>
+   Copyright (C) 2009-2011, Stefan Hacker <dd0t@users.sourceforge.net>
 
    All rights reserved.
 
@@ -29,12 +29,14 @@
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
-#ifndef _LOG_H
-#define _LOG_H
+#ifndef LOG_H_
+#define LOG_H_
+
+#include <QtCore/QDate>
+#include <QtGui/QTextCursor>
+#include <QtGui/QTextDocument>
 
-#include "mumble_pch.hpp"
 #include "ConfigDialog.h"
-#include "AudioOutput.h"
 #include "ui_Log.h"
 
 class TextToSpeech;
@@ -69,7 +71,7 @@ class Log : public QObject {
 		Q_OBJECT
 		Q_DISABLE_COPY(Log)
 	public:
-		enum MsgType { DebugInfo, CriticalError, Warning, Information, ServerConnected, ServerDisconnected, UserJoin, UserLeave, YouKicked, UserKicked, SelfMute, OtherSelfMute, YouMuted, YouMutedOther, OtherMutedOther, ChannelJoin, ChannelLeave, PermissionDenied, TextMessage };
+		enum MsgType { DebugInfo, CriticalError, Warning, Information, ServerConnected, ServerDisconnected, UserJoin, UserLeave, Recording, YouKicked, UserKicked, SelfMute, OtherSelfMute, YouMuted, YouMutedOther, OtherMutedOther, ChannelJoin, ChannelLeave, PermissionDenied, TextMessage };
 		enum LogColorType { Time, Server, Privilege, Source, Target };
 		static const MsgType firstMsgType = DebugInfo;
 		static const MsgType lastMsgType = TextMessage;
@@ -81,6 +83,8 @@ class Log : public QObject {
 		unsigned int uiLastId;
 		QDate qdDate;
 		static const QStringList allowedSchemes();
+		void postNotification(MsgType mt, const QString &console, const QString &plain);
+		void postQtNotification(MsgType mt, const QString &plain);
 	public:
 		Log(QObject *p = NULL);
 		QString msgName(MsgType t) const;
@@ -93,7 +97,7 @@ class Log : public QObject {
 		static QString formatClientUser(ClientUser *cu, LogColorType t);
 		static QString formatChannel(::Channel *c);
 	public slots:
-		void log(MsgType t, const QString &console, const QString &terse=QString());
+		void log(MsgType t, const QString &console, const QString &terse=QString(), bool ownMessage = false);
 };
 
 class ValidDocument : public QTextDocument {
@@ -121,6 +125,4 @@ class LogDocument : public QTextDocument {
 		void finished();
 };
 
-#else
-class Log;
 #endif
