@@ -1,5 +1,5 @@
-/* Copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
-   Copyright (C) 2009, Stefan Hacker <dd0t@users.sourceforge.net>
+/* Copyright (C) 2005-2011, Thorvald Natvig <thorvald@natvig.com>
+   Copyright (C) 2009-2011, Stefan Hacker <dd0t@users.sourceforge.net>
 
    All rights reserved.
 
@@ -29,12 +29,20 @@
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
-#ifndef _MAINWINDOW_H
-#define _MAINWINDOW_H
+#ifndef MAINWINDOW_H_
+#define MAINWINDOW_H_
 
-#include "mumble_pch.hpp"
+#include <QtGui/QMainWindow>
+#include <QtGui/QSystemTrayIcon>
+#include <QtNetwork/QAbstractSocket>
+
+#include "CustomElements.h"
+#include "Message.h"
+#include "Mumble.pb.h"
+#include "Usage.h"
+
+#include "ui_MainWindow.h"
 
-#define TI_QEVENT (QEvent::User + 938)
 #define MB_QEVENT (QEvent::User + 939)
 #define OU_QEVENT (QEvent::User + 940)
 
@@ -48,14 +56,11 @@ class UserModel;
 class Tokens;
 class Channel;
 class UserInformation;
+class VoiceRecorderDialog;
+class PTTButtonWidget;
 
 struct ShortcutTarget;
 
-#include "Message.h"
-#include "Usage.h"
-#include "ui_MainWindow.h"
-#include "CustomElements.h"
-
 class MessageBoxEvent : public QEvent {
 	public:
 		QString msg;
@@ -80,11 +85,11 @@ class MainWindow : public QMainWindow, public MessageHandler, public Ui::MainWin
 		QMenu *qmChannel;
 		QMenu *qmTray;
 		QIcon qiIcon, qiIconMuteSelf, qiIconMuteServer, qiIconDeafSelf, qiIconDeafServer, qiIconMuteSuppressed;
-		QIcon qiTalkingOn, qiTalkingWhisper, qiTalkingWhisperChannel, qiTalkingOff;
+		QIcon qiTalkingOn, qiTalkingWhisper, qiTalkingShout, qiTalkingOff;
 
 		GlobalShortcut *gsPushTalk, *gsResetAudio, *gsMuteSelf, *gsDeafSelf;
 		GlobalShortcut *gsUnlink, *gsPushMute, *gsMetaChannel, *gsToggleOverlay;
-		GlobalShortcut *gsMinimal, *gsVolumeUp, *gsVolumeDown, *gsWhisper;
+		GlobalShortcut *gsMinimal, *gsVolumeUp, *gsVolumeDown, *gsWhisper, *gsMetaLink;
 		DockTitleBar *dtbLogDockTitle, *dtbChatDockTitle;
 
 		ACLEditor *aclEdit;
@@ -92,23 +97,31 @@ class MainWindow : public QMainWindow, public MessageHandler, public Ui::MainWin
 		UserEdit *userEdit;
 		Tokens *tokenEdit;
 
+		VoiceRecorderDialog *voiceRecorderDialog;
+
 		MumbleProto::Reject_RejectType rtLast;
 		QString qsDesiredChannel;
 
 		bool bSuppressAskOnQuit;
 		bool bAutoUnmute;
 
-		unsigned int uiContextSession;
-		int iContextChannel;
+		QWeakPointer<Channel> cContextChannel;
+		QWeakPointer<ClientUser> cuContextUser;
+		QPoint qpContextPosition;
 
 		void recheckTTS();
 		void msgBox(QString msg);
 		void setOnTop(bool top);
+		void setShowDockTitleBars(bool doShow);
 		void updateTrayIcon();
 		QPair<QByteArray, QImage> openImageFile();
 		static const QString defaultStyleSheet;
 
+		void updateChatBar();
+		void openTextMessageDialog(ClientUser *p);
+
 #ifdef Q_OS_WIN
+		Timer tInactive;
 		bool winEvent(MSG *, long *);
 		unsigned int uiNewHardware;
 #endif
@@ -127,23 +140,24 @@ class MainWindow : public QMainWindow, public MessageHandler, public Ui::MainWin
 		int iTargetCounter;
 		QMap<unsigned int, UserInformation *> qmUserInformations;
 
+		PTTButtonWidget *qwPTTButtonWidget;
+
 		void createActions();
 		void setupGui();
 		void customEvent(QEvent *evt);
 		void findDesiredChannel();
 		void setupView(bool toggle_minimize = true);
-		bool launchCompatibilityClient(const QUrl &url);
-		bool bNoHide;
 		virtual void closeEvent(QCloseEvent *e);
 		virtual void hideEvent(QHideEvent *e);
+		virtual void showEvent(QShowEvent *e);
 
-		bool handleSpecialContextMenu(const QUrl &url, const QPoint &_pos, bool focus = false);
+		bool handleSpecialContextMenu(const QUrl &url, const QPoint &pos_, bool focus = false);
 		Channel* getContextMenuChannel();
 		ClientUser* getContextMenuUser();
 
 	public slots:
 		void on_qmServer_aboutToShow();
-		void on_qaServerConnect_triggered();
+		void on_qaServerConnect_triggered(bool autoconnect = false);
 		void on_qaServerDisconnect_triggered();
 		void on_qaServerBanList_triggered();
 		void on_qaServerUserList_triggered();
@@ -161,6 +175,9 @@ class MainWindow : public QMainWindow, public MessageHandler, public Ui::MainWin
 		void on_qaUserBan_triggered();
 		void on_qaUserMute_triggered();
 		void on_qaUserDeaf_triggered();
+		void on_qaSelfPrioritySpeaker_triggered();
+		void on_qaUserPrioritySpeaker_triggered();
+		void on_qaUserLocalIgnore_triggered();
 		void on_qaUserLocalMute_triggered();
 		void on_qaUserTextMessage_triggered();
 		void on_qaUserRegister_triggered();
@@ -177,9 +194,11 @@ class MainWindow : public QMainWindow, public MessageHandler, public Ui::MainWin
 		void on_qaChannelUnlink_triggered();
 		void on_qaChannelUnlinkAll_triggered();
 		void on_qaChannelSendMessage_triggered();
+		void on_qaChannelCopyURL_triggered();
 		void on_qaAudioReset_triggered();
 		void on_qaAudioMute_triggered();
 		void on_qaAudioDeaf_triggered();
+		void on_qaRecording_triggered();
 		void on_qaAudioTTS_triggered();
 		void on_qaAudioUnlink_triggered();
 		void on_qaAudioStats_triggered();
@@ -191,13 +210,12 @@ class MainWindow : public QMainWindow, public MessageHandler, public Ui::MainWin
 		void on_qaAudioWizard_triggered();
 		void on_qaHelpWhatsThis_triggered();
 		void on_qaHelpAbout_triggered();
-		void on_qaHelpAboutSpeex_triggered();
 		void on_qaHelpAboutQt_triggered();
 		void on_qaHelpVersionCheck_triggered();
 		void on_qaQuit_triggered();
-		void on_qleChat_returnPressed();
-		void on_qleChat_tabPressed();
-		void on_qleChat_ctrlSpacePressed();
+		void on_qteChat_tabPressed();
+		void on_qteChat_ctrlSpacePressed();
+		void on_qtvUsers_customContextMenuRequested(const QPoint &mpos);
 		void on_qteLog_customContextMenuRequested(const QPoint &pos);
 		void on_qteLog_anchorClicked(const QUrl &);
 		void on_qteLog_highlighted(const QUrl & link);
@@ -211,7 +229,9 @@ class MainWindow : public QMainWindow, public MessageHandler, public Ui::MainWin
 		void on_gsDeafSelf_down(QVariant);
 		void on_gsWhisper_triggered(bool, QVariant);
 		void on_Reconnect_timeout();
+		void on_Icon_messageClicked();
 		void on_Icon_activated(QSystemTrayIcon::ActivationReason);
+		void voiceRecorderDialog_finished(int);
 		void qtvUserCurrentChanged(const QModelIndex &, const QModelIndex &);
 		void serverConnected();
 		void serverDisconnected(QAbstractSocket::SocketError, QString reason);
@@ -223,6 +243,10 @@ class MainWindow : public QMainWindow, public MessageHandler, public Ui::MainWin
 		void talkingChanged();
 		void destroyUserInformation();
 		void trayAboutToShow();
+		void sendChatbarMessage(QString msg);
+		void pttReleased();
+		void whisperReleased(QVariant scdata);
+		void onResetAudio();
 	public:
 		MainWindow(QWidget *parent);
 		~MainWindow();
@@ -231,8 +255,7 @@ class MainWindow : public QMainWindow, public MessageHandler, public Ui::MainWin
 #define MUMBLE_MH_MSG(x) void msg##x(const MumbleProto:: x &);
 		MUMBLE_MH_ALL
 #undef MUMBLE_MH_MSG
+		void removeContextAction(const MumbleProto::ContextActionModify &msg);
 };
 
-#else
-class MainWindow;
 #endif
