@@ -1,4 +1,4 @@
-/* copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
+/* copyright (C) 2005-2011, Thorvald Natvig <thorvald@natvig.com>
 
    All rights reserved.
 
@@ -28,10 +28,13 @@
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
+#include "mumble_pch.hpp"
+
 #include "RichTextEditor.h"
+
 #include "Global.h"
-#include "MainWindow.h"
 #include "Log.h"
+#include "MainWindow.h"
 
 RichTextHtmlEdit::RichTextHtmlEdit(QWidget *p) : QTextEdit(p) {
 }
@@ -130,7 +133,7 @@ void RichTextHtmlEdit::insertFromMimeData(const QMimeData *source) {
 		QByteArray qba = source->data(QLatin1String("application/x-qt-windows-mime;value=\"FileGroupDescriptorW\""));
 		if (qba.length() == sizeof(FILEGROUPDESCRIPTORW)) {
 			const FILEGROUPDESCRIPTORW *ptr = reinterpret_cast<const FILEGROUPDESCRIPTORW *>(qba.constData());
-			title = QString::fromUtf16(ptr->fgd[0].cFileName);
+			title = QString::fromWCharArray(ptr->fgd[0].cFileName);
 			if (title.endsWith(QLatin1String(".url"), Qt::CaseInsensitive))
 				title = title.left(title.length() - 4);
 		}
@@ -237,8 +240,8 @@ void RichTextEditor::on_qaImage_triggered() {
 	if (qba.isEmpty())
 		return;
 
-	if (qba.length() > 65536) {
-		QMessageBox::warning(this, tr("Failed to load image"), tr("Image file too large to embed in document. Please use images smaller than %1 kB.").arg(65536/1024));
+	if ((g.uiImageLength > 0) && (qba.length() > g.uiImageLength)) {
+		QMessageBox::warning(this, tr("Failed to load image"), tr("Image file too large to embed in document. Please use images smaller than %1 kB.").arg(g.uiImageLength /1024));
 		return;
 	}
 
@@ -436,10 +439,16 @@ static void recurseParse(QXmlStreamReader &reader, QXmlStreamWriter &writer, int
 							writer.writeAttribute(QLatin1String("style"), qsl.join(QLatin1String("; ")));
 						}
 					} else if (name == QLatin1String("p")) {
-						// Ignore first paragraph.
-
 						paragraphs++;
-						if (paragraphs > 1) {
+						if (paragraphs == 1) {
+							// Ignore first paragraph. If it is styled empty drop its contents (e.g. <br />) too.
+							if (style.value(QLatin1String("-qt-paragraph-type")) == QLatin1String("empty")) {
+								reader.skipCurrentElement();
+								continue;
+							}
+							rclose = 0;
+						}
+						else {
 							rclose = 1;
 							writer.writeStartElement(name);
 
@@ -455,8 +464,6 @@ static void recurseParse(QXmlStreamReader &reader, QXmlStreamWriter &writer, int
 
 								writer.writeAttribute(QLatin1String("style"), qsl.join(QLatin1String("; ")));
 							}
-						} else {
-							rclose = 0;
 						}
 					} else if (name == QLatin1String("a")) {
 						// Set pstyle to include implicit blue underline.
