@@ -1,5 +1,5 @@
-/* Copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
-   Copyright (C) 2009, Stefan Hacker <dd0t@users.sourceforge.net>
+/* Copyright (C) 2005-2011, Thorvald Natvig <thorvald@natvig.com>
+   Copyright (C) 2009-2011, Stefan Hacker <dd0t@users.sourceforge.net>
 
    All rights reserved.
 
@@ -29,10 +29,19 @@
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
-#ifndef _SETTINGS_H
-#define _SETTINGS_H
+#ifndef SETTINGS_H_
+#define SETTINGS_H_
 
-#include "mumble_pch.hpp"
+#include <QtCore/QVariant>
+#include <QtCore/QList>
+#include <QtCore/QPair>
+#include <QtCore/QRectF>
+#include <QtCore/QSettings>
+#include <QtCore/QStringList>
+#include <QtGui/QColor>
+#include <QtGui/QFont>
+#include <QtNetwork/QSslCertificate>
+#include <QtNetwork/QSslKey>
 
 // Global helper classes to spread variables around across threads
 // especially helpfull to initialize things like the stored
@@ -71,38 +80,119 @@ quint32 qHash(const QList<ShortcutTarget> &);
 
 QDataStream &operator<<(QDataStream &, const ShortcutTarget &);
 QDataStream &operator>>(QDataStream &, ShortcutTarget &);
-Q_DECLARE_METATYPE(ShortcutTarget);
+Q_DECLARE_METATYPE(ShortcutTarget)
+
+struct OverlaySettings {
+	enum OverlayPresets { AvatarAndName, LargeSquareAvatar };
+
+	enum OverlayShow { Talking, Active, HomeChannel, LinkedChannels };
+
+	enum OverlaySort { Alphabetical, LastStateChange };
+
+	bool bEnable;
+
+	QString qsStyle;
+
+	OverlayShow osShow;
+	bool bAlwaysSelf;
+	int uiActiveTime; // Time in seconds for a user to stay active after talking
+	OverlaySort osSort;
+
+	float fX;
+	float fY;
+
+	qreal fZoom;
+	unsigned int uiColumns;
+
+	QColor qcUserName[4];
+	QFont qfUserName;
+
+	QColor qcChannel;
+	QFont qfChannel;
+
+	QColor qcFps;
+	QFont qfFps;
+
+	qreal fBoxPad;
+	qreal fBoxPenWidth;
+	QColor qcBoxPen;
+	QColor qcBoxFill;
+
+	bool bUserName;
+	bool bChannel;
+	bool bMutedDeafened;
+	bool bAvatar;
+	bool bBox;
+	bool bFps;
+
+	qreal fUserName;
+	qreal fChannel;
+	qreal fMutedDeafened;
+	qreal fAvatar;
+	qreal fUser[4];
+	qreal fFps;
+
+	QRectF qrfUserName;
+	QRectF qrfChannel;
+	QRectF qrfMutedDeafened;
+	QRectF qrfAvatar;
+	QRectF qrfFps;
+
+	Qt::Alignment qaUserName;
+	Qt::Alignment qaChannel;
+	Qt::Alignment qaMutedDeafened;
+	Qt::Alignment qaAvatar;
+
+	bool bUseWhitelist;
+	QStringList qslBlacklist;
+	QStringList qslWhitelist;
+
+	OverlaySettings();
+	void setPreset(const OverlayPresets preset = AvatarAndName);
+
+	void load();
+	void load(QSettings*);
+	void save();
+	void save(QSettings*);
+};
 
 struct Settings {
 	enum AudioTransmit { Continous, VAD, PushToTalk };
 	enum VADSource { Amplitude, SignalToNoise };
 	enum LoopMode { None, Local, Server };
-	enum OverlayShow { Nothing, Talking, All };
 	enum ChannelExpand { NoChannels, ChannelsWithUsers, AllChannels };
 	enum ChannelDrag { Ask, DoNothing, Move };
 	enum ServerShow { ShowPopulated, ShowReachable, ShowAll };
+	enum TalkState { Passive, Talking, Whispering, Shouting };
+	enum IdleAction { Nothing, Deafen, Mute };
 	typedef QPair<QList<QSslCertificate>, QSslKey> KeyPair;
 
 	AudioTransmit atTransmit;
 	quint64 uiDoublePush;
+	quint64 uiPTTHold;
 
 	bool bExpert;
 
-	bool bPushClick;
+	bool bTxAudioCue;
 	static const QString cqsDefaultPushClickOn;
 	static const QString cqsDefaultPushClickOff;
-	QString qsPushClickOn;
-	QString qsPushClickOff;
+	QString qsTxAudioCueOn;
+	QString qsTxAudioCueOff;
 
 	bool bTransmitPosition;
 	bool bMute, bDeaf;
 	bool bTTS;
 	bool bUserTop;
 	bool bWhisperFriends;
+	bool bTTSMessageReadBack;
 	int iTTSVolume, iTTSThreshold;
 	int iQuality, iMinLoudness, iVoiceHold, iJitterBufferSize;
 	int iNoiseSuppress;
+
+	// Idle auto actions
 	unsigned int iIdleTime;
+	IdleAction iaeIdleAction;
+
 	VADSource vsVAD;
 	float fVADmin, fVADmax;
 	int iFramesPerPacket;
@@ -124,6 +214,7 @@ struct Settings {
 	QString qsWASAPIInput, qsWASAPIOutput;
 	QByteArray qbaDXInput, qbaDXOutput;
 
+	bool bExclusiveInput, bExclusiveOutput;
 	bool bEcho;
 	bool bEchoMulti;
 	bool bPositionalAudio;
@@ -131,34 +222,24 @@ struct Settings {
 	float fAudioMinDistance, fAudioMaxDistance, fAudioMaxDistVolume, fAudioBloom;
 	QMap<QString, bool> qmPositionalAudioPlugins;
 
-	bool bOverlayEnable;
-	OverlayShow osOverlay;
-	bool bOverlayAlwaysSelf;
-	float fOverlayX;
-	float fOverlayY;
-	bool bOverlayLeft, bOverlayRight, bOverlayTop, bOverlayBottom;
-	QFont qfOverlayFont;
-	float fOverlayHeight;
-	QColor qcOverlayUser;
-	QColor qcOverlayTalking;
-	QColor qcOverlayWhisper;
-	QColor qcOverlayChannel;
-	QColor qcOverlayChannelTalking;
+	OverlaySettings os;
 
 	int iLCDUserViewMinColWidth;
 	int iLCDUserViewSplitterWidth;
 	QMap<QString, bool> qmLCDDevices;
 
+	bool bShortcutEnable;
+	bool bSuppressMacEventTapWarning;
 	QList<Shortcut> qlShortcuts;
 
 	enum MessageLog { LogNone = 0x00, LogConsole = 0x01, LogTTS = 0x02, LogBalloon = 0x04, LogSoundfile = 0x08};
+	int iMaxLogBlocks;
 	QMap<int, QString> qmMessageSounds;
 	QMap<int, quint32> qmMessages;
 
 	QString qsLanguage;
 	QString qsStyle;
 	QString qsSkin;
-	bool bFirstTime;
 	QByteArray qbaMainWindowGeometry, qbaMainWindowState, qbaMinimalViewGeometry, qbaMinimalViewState, qbaSplitterState, qbaHeaderState;
 	QByteArray qbaConfigGeometry;
 	enum WindowLayout { LayoutClassic, LayoutStacked, LayoutHybrid, LayoutCustom };
@@ -170,10 +251,11 @@ struct Settings {
 	enum AlwaysOnTopBehaviour { OnTopNever, OnTopAlways, OnTopInMinimal, OnTopInNormal };
 	AlwaysOnTopBehaviour aotbAlwaysOnTop;
 	bool bAskOnQuit;
-	bool bHideTray;
+	bool bHideInTray;
 	bool bStateInTray;
 	bool bUsage;
 	bool bShowUserCount;
+	bool bChatBarUseSelection;
 	QByteArray qbaConnectDialogHeader, qbaConnectDialogGeometry;
 	bool bShowContextMenuInMenuBar;
 
@@ -186,14 +268,20 @@ struct Settings {
 	bool bUpdateCheck;
 	bool bPluginCheck;
 
+	// PTT Button window
+	bool bShowPTTButtonWindow;
+	QByteArray qbaPTTButtonWindowGeometry;
+
 	// Network settings
 	enum ProxyType { NoProxy, HttpProxy, Socks5Proxy };
 	bool bTCPCompat;
 	bool bReconnect;
+	bool bAutoConnect;
 	bool bQoS;
 	ProxyType ptProxyType;
 	QString qsProxyHost, qsProxyUsername, qsProxyPassword;
 	unsigned short usProxyPort;
+	QString qsRegionalHost;
 
 	static const int ciDefaultMaxImageSize = 50 * 1024; // Restrict to 50KiB as a default
 	int iMaxImageSize;
@@ -205,6 +293,19 @@ struct Settings {
 	// Accessibility
 	bool bHighContrast;
 
+	// Recording
+	QString qsRecordingPath;
+	QString qsRecordingFile;
+	enum RecordingMode { RecordingMixdown, RecordingMultichannel };
+	RecordingMode rmRecordingMode;
+	int iRecordingFormat;
+
+	// Codec kill-switch
+	bool bDisableCELT;
+
+	// Config updates
+	unsigned int uiUpdateCounter;
+
 	// Nonsaved
 	LoopMode lmLoopMode;
 	float dPacketLoss;
@@ -215,9 +316,8 @@ struct Settings {
 
 	Settings();
 	void load();
+	void load(QSettings*);
 	void save();
 };
 
-#else
-struct Settings;
 #endif
