@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2009, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2011, Thorvald Natvig <thorvald@natvig.com>
 
    All rights reserved.
 
@@ -28,10 +28,9 @@
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
-#ifndef _TEXTMESSAGE_H
-#define _TEXTMESSAGE_H
+#ifndef TEXTMESSAGE_H_
+#define TEXTMESSAGE_H_
 
-#include "mumble_pch.hpp"
 #include "ui_TextMessage.h"
 
 class TextMessage : public QDialog, public Ui::TextMessage {
@@ -40,14 +39,14 @@ class TextMessage : public QDialog, public Ui::TextMessage {
 		Q_DISABLE_COPY(TextMessage)
 	protected:
 		QString qsRep;
+	public slots:
+		void on_qcbTreeMessage_stateChanged(int);
 	public:
-		TextMessage(QWidget *parent = NULL);
+		TextMessage(QWidget *parent = NULL, QString title = tr("Enter text"), bool bChannel = false);
 		QString message();
 		bool eventFilter(QObject *obj, QEvent *event);
 		static QString autoFormat(QString qsPlain);
-	public slots:
-		void on_qteEdit_textChanged();
-		void on_qcbRawMessage_stateChanged(int);
+		bool bTreeMessage;
 };
 
 #endif
