@@ -1,4 +1,4 @@
-/* copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
+/* copyright (C) 2005-2011, Thorvald Natvig <thorvald@natvig.com>
 
    All rights reserved.
 
@@ -28,9 +28,13 @@
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
+#include "mumble_pch.hpp"
+
 #include "UserEdit.h"
-#include "ServerHandler.h"
+
 #include "Global.h"
+#include "ServerHandler.h"
+#include "User.h"
 
 UserEdit::UserEdit(const MumbleProto::UserList &msg, QWidget *p) : QDialog(p) {
 	setupUi(this);
@@ -42,10 +46,9 @@ UserEdit::UserEdit(const MumbleProto::UserList &msg, QWidget *p) : QDialog(p) {
 		int id = u.user_id();
 		const QString &name = u8(u.name());
 
-		QListWidgetItem *qlwi = new QListWidgetItem(name);
-		qlwi->setFlags(qlwi->flags() | Qt::ItemIsEditable);
-		qlwi->setData(Qt::UserRole, id);
-		qlwUserList->addItem(qlwi);
+		UserEditListItem *ueli = new UserEditListItem(name, id);
+
+		qlwUserList->addItem(ueli);
 		qmUsers.insert(id, name);
 	}
 }
@@ -106,3 +109,16 @@ void UserEdit::renameTriggered() {
 		qlwUserList->editItem(item);
 	}
 }
+
+UserEditListItem::UserEditListItem(const QString &username, const int userid) : QListWidgetItem(username) {
+	setFlags(flags() | Qt::ItemIsEditable);
+	setData(Qt::UserRole, userid);
+}
+
+bool UserEditListItem::operator<(const QListWidgetItem &other) const {
+	// Avoid duplicating the User sorting code for a little more complexity
+	User first, second;
+	first.qsName = text();
+	second.qsName = other.text();
+	return User::lessThan(&first, &second);
+}
