@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2011, Thorvald Natvig <thorvald@natvig.com>
 
    All rights reserved.
 
@@ -28,6 +28,8 @@
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
+#include "mumble_pch.hpp"
+
 #include "Overlay.h"
 #include "MainWindow.h"
 #include "ServerHandler.h"
@@ -55,22 +57,37 @@
 #include "FileEngine.h"
 #include "SocketRPC.h"
 
+#ifdef USE_STATIC
+// Keep in sync with mumble.pro QTPLUGIN list.
+Q_IMPORT_PLUGIN(qtaccessiblewidgets)
+Q_IMPORT_PLUGIN(qico)
+Q_IMPORT_PLUGIN(qsvg)
+Q_IMPORT_PLUGIN(qsvgicon)
+#ifdef Q_OS_MAC
+Q_IMPORT_PLUGIN(qicnsicon)
+#endif
+#endif
+
 #ifdef BOOST_NO_EXCEPTIONS
 namespace boost {
-void throw_exception(std::exception const &) {
-	qFatal("Boost exception caught!");
-}
+	void throw_exception(std::exception const &) {
+		qFatal("Boost exception caught!");
+	}
 }
 #endif
 
 extern void os_init();
-extern char *os_url;
 extern char *os_lang;
 
-class QAppMumble:public QApplication {
+class QAppMumble : public QApplication {
 	public:
+		QUrl quLaunchURL;
 		QAppMumble(int &pargc, char **pargv) : QApplication(pargc, pargv) {}
 		void commitData(QSessionManager&);
+		bool event(QEvent *e);
+#ifdef Q_OS_WIN
+		bool winEventFilter(MSG *msg, long *result);
+#endif
 };
 
 void QAppMumble::commitData(QSessionManager &) {
@@ -81,12 +98,54 @@ void QAppMumble::commitData(QSessionManager &) {
 	}
 }
 
-int main(int argc, char **argv) {
-	int res;
+bool QAppMumble::event(QEvent *e) {
+	if (e->type() == QEvent::FileOpen) {
+		QFileOpenEvent *foe = static_cast<QFileOpenEvent *>(e);
+		if (! g.mw) {
+			this->quLaunchURL = foe->url();
+		} else {
+			g.mw->openUrl(foe->url());
+		}
+		return true;
+	}
+	return QApplication::event(e);
+}
 
+#ifdef Q_OS_WIN
+bool QAppMumble::winEventFilter(MSG *msg, long *result) {
+	if (QThread::currentThread() == thread()) {
+		if (Global::g_global_struct && g.ocIntercept) {
+			switch (msg->message) {
+				case WM_MOUSELEAVE:
+					*result = 0;
+					return true;
+				case WM_KEYDOWN:
+				case WM_KEYUP:
+				case WM_SYSKEYDOWN:
+				case WM_SYSKEYUP:
+					GlobalShortcutEngine::engine->prepareInput();
+				default:
+					break;
+			}
+		}
+	}
+	return QApplication::winEventFilter(msg, result);
+}
+#endif
+
+int main(int argc, char **argv) {
+	int res = 0;
 
 	QT_REQUIRE_VERSION(argc, argv, "4.4.0");
 
+#if defined(Q_OS_WIN)
+	SetDllDirectory(L"");
+#else
+#ifndef Q_OS_MAC
+	setenv("AVAHI_COMPAT_NOWARN", "1", 1);
+#endif
+#endif
+
 	// Initialize application object.
 	QAppMumble a(argc, argv);
 	a.setApplicationName(QLatin1String("Mumble"));
@@ -94,6 +153,15 @@ int main(int argc, char **argv) {
 	a.setOrganizationDomain(QLatin1String("mumble.sourceforge.net"));
 	a.setQuitOnLastWindowClosed(false);
 
+	#ifdef USE_SBCELT
+	{
+		// For now, force Mumble to use sbcelt-helper from the same directory as the 'mumble' executable.
+		QDir d(a.applicationDirPath());
+		QString helper = d.absoluteFilePath(QString::fromLatin1("sbcelt-helper"));
+		setenv("SBCELT_HELPER_BINARY", helper.toUtf8().constData(), 1);
+	}
+#endif
+
 	Global::g_global_struct = new Global();
 
 	qsrand(QDateTime::currentDateTime().toTime_t());
@@ -107,8 +175,37 @@ int main(int argc, char **argv) {
 	if (a.arguments().count() > 1) {
 		QStringList args = a.arguments();
 		for (int i = 1; i < args.count(); ++i) {
-			if (args.at(i) == QLatin1String("-m")) {
+			if (args.at(i) == QLatin1String("-h") || args.at(i) == QLatin1String("--help")
+#if defined(Q_OS_WIN)
+				|| args.at(i) == QLatin1String("/?")
+#endif
+			) {
+				QString helpmessage = MainWindow::tr( "Usage: mumble [options] [<url>]\n"
+					"\n"
+					"<url> specifies a URL to connect to after startup instead of showing\n"
+					"the connection window, and has the following form:\n"
+					"mumble://[<username>[:<password>]@]<host>[:<port>][/<channel>[/<subchannel>...]][?version=<x.y.z>]\n"
+					"\n"
+					"The version query parameter has to be set in order to invoke the\n"
+					"correct client version. It currently defaults to 1.2.0.\n"
+					"\n"
+					"Valid options are:\n"
+					"  -h, --help    Show this help text and exit.\n"
+					"  -m, --multiple\n"
+					"                Allow multiple instances of the client to be started.\n"
+					"  -n, --noidentity\n"
+					"                Suppress loading of identity files (i.e., certificates.)\n"
+					);
+#if defined(Q_OS_WIN)
+				QMessageBox::information(NULL, MainWindow::tr("Invocation"), helpmessage);
+#else
+				printf("%s", qPrintable(helpmessage));
+#endif
+				return 1;
+			} else if (args.at(i) == QLatin1String("-m") || args.at(i) == QLatin1String("--multiple")) {
 				bAllowMultiple = true;
+			} else if (args.at(i) == QLatin1String("-n") || args.at(i) == QLatin1String("--noidentity")) {
+				g.s.bSuppressIdentity = true;
 			} else {
 				QUrl u = QUrl::fromEncoded(args.at(i).toUtf8());
 				if (u.isValid() && (u.scheme() == QLatin1String("mumble"))) {
@@ -145,61 +242,21 @@ int main(int argc, char **argv) {
 
 	if (! bAllowMultiple) {
 		if (url.isValid()) {
-			int major, minor, patch;
-			major = 1;
-			minor = 1;
-			patch = 0;
-
-			QString version = url.queryItemValue(QLatin1String("version"));
 #ifndef USE_DBUS
 			QMap<QString, QVariant> param;
 			param.insert(QLatin1String("href"), url);
 #endif
-
-			QRegExp rx(QLatin1String("(\\d+)\\.(\\d+)\\.(\\d+)"));
-			if (rx.exactMatch(version)) {
-				major = rx.cap(1).toInt();
-				minor = rx.cap(2).toInt();
-				patch = rx.cap(3).toInt();
-			}
-
-			if ((major == 1) && (minor == 1)) {
-				bool sent = false;
+			bool sent = false;
 #ifdef USE_DBUS
-				QDBusInterface qdbi(QLatin1String("net.sourceforge.mumble.mumble11x"), QLatin1String("/"), QLatin1String("net.sourceforge.mumble.Mumble"));
+			QDBusInterface qdbi(QLatin1String("net.sourceforge.mumble.mumble"), QLatin1String("/"), QLatin1String("net.sourceforge.mumble.Mumble"));
 
-				QDBusMessage reply=qdbi.call(QLatin1String("openUrl"), QLatin1String(url.toEncoded()));
-				sent = (reply.type() == QDBusMessage::ReplyMessage);
+			QDBusMessage reply=qdbi.call(QLatin1String("openUrl"), QLatin1String(url.toEncoded()));
+			sent = (reply.type() == QDBusMessage::ReplyMessage);
 #else
-				sent = SocketRPC::send(QLatin1String("Mumble11x"), QLatin1String("url"), param);
+			sent = SocketRPC::send(QLatin1String("Mumble"), QLatin1String("url"), param);
 #endif
-				if (sent) {
-					return 0;
-				} else {
-					QString executable = a.applicationFilePath();
-					int idx = executable.lastIndexOf(QLatin1String("mumble"));
-					if (idx >= 0) {
-						QStringList args;
-						args << url.toString();
-
-						executable.replace(idx, 6, QLatin1String("mumble11x"));
-						if (QProcess::startDetached(executable, args))
-							return 0;
-					}
-				}
-			} else {
-				bool sent = false;
-#ifdef USE_DBUS
-				QDBusInterface qdbi(QLatin1String("net.sourceforge.mumble.mumble"), QLatin1String("/"), QLatin1String("net.sourceforge.mumble.Mumble"));
-
-				QDBusMessage reply=qdbi.call(QLatin1String("openUrl"), QLatin1String(url.toEncoded()));
-				sent = (reply.type() == QDBusMessage::ReplyMessage);
-#else
-				sent = SocketRPC::send(QLatin1String("Mumble"), QLatin1String("url"), param);
-#endif
-				if (sent)
-					return 0;
-			}
+			if (sent)
+				return 0;
 		} else {
 			bool sent = false;
 #ifdef USE_DBUS
@@ -219,7 +276,7 @@ int main(int argc, char **argv) {
 	// Load preferences
 	g.s.load();
 
-	// Check wheter we need to enable accessbility features
+	// Check wheter we need to enable accessibility features
 #ifdef Q_OS_WIN
 	// Only windows for now. Could not find any information on how to query this for osx or linux
 	{
@@ -237,6 +294,7 @@ int main(int argc, char **argv) {
 
 	if (! g.s.qsStyle.isEmpty()) {
 		a.setStyle(g.s.qsStyle);
+		g.qsCurrentStyle = g.s.qsStyle;
 	}
 
 	if (! g.s.qsSkin.isEmpty()) {
@@ -260,13 +318,12 @@ int main(int argc, char **argv) {
 #ifdef Q_OS_MAC
 	if (os_lang) {
 		qWarning("Using Mac OS X system langauge as locale name");
-		qsSystemLocale = QString(os_lang);
+		qsSystemLocale = QLatin1String(os_lang);
 	}
 #endif
 
-	qWarning("Locale is %s", qPrintable(qsSystemLocale));
-
-	QString locale = g.s.qsLanguage.isEmpty() ? qsSystemLocale : g.s.qsLanguage;
+	const QString locale = g.s.qsLanguage.isEmpty() ? qsSystemLocale : g.s.qsLanguage;
+	qWarning("Locale is \"%s\" (System: \"%s\")", qPrintable(locale), qPrintable(qsSystemLocale));
 
 	QTranslator translator;
 	if (translator.load(QLatin1String("translation:mumble_") + locale))
@@ -282,8 +339,10 @@ int main(int argc, char **argv) {
 	else if (qttranslator.load(QLatin1String("translation:qt_") + locale))
 		a.installTranslator(&qttranslator);
 
-	g.qsRegionalHost = qsSystemLocale;
-	g.qsRegionalHost = g.qsRegionalHost.remove(QRegExp(QLatin1String("^.+_"))).toLower() + QLatin1String(".mumble.info");
+	if (g.s.qsRegionalHost.isEmpty()) {
+		g.s.qsRegionalHost = qsSystemLocale;
+		g.s.qsRegionalHost = g.s.qsRegionalHost.remove(QRegExp(QLatin1String("^.+_"))).toLower() + QLatin1String(".mumble.info");
+	}
 
 	// Initialize proxy settings
 	NetworkConfig::SetupProxy();
@@ -307,20 +366,14 @@ int main(int argc, char **argv) {
 	g.bc = new BonjourClient();
 #endif
 
-	// Initialize the serverhandler
-	g.sh = new ServerHandler();
-	g.sh->moveToThread(g.sh);
-
 	g.o = new Overlay();
-	g.o->setActive(g.s.bOverlayEnable);
+	g.o->setActive(g.s.os.bEnable);
 
 	g.lcd = new LCD();
 
 	// Process any waiting events before initializing our MainWindow.
 	// The mumble:// URL support for Mac OS X happens through AppleEvents,
-	// so we need to loop a little before we begin.  If we were launched
-	// through a URL, this should call a platform specific callback
-	// (in os_macx.cpp) and point the `os_url' global to a valid URL.
+	// so we need to loop a little before we begin.
 	a.processEvents();
 
 	// Main Window
@@ -343,25 +396,35 @@ int main(int argc, char **argv) {
 
 	MumbleFileEngineHandler *mfeh = new MumbleFileEngineHandler();
 
-	// And the start the last chosen audio system.
-	g.ai = AudioInputRegistrar::newFromChoice();
-	if (g.ai)
-		g.ai->start(QThread::HighestPriority);
-	g.ao = AudioOutputRegistrar::newFromChoice();
-	if (g.ao)
-		g.ao->start(QThread::HighPriority);
+	Audio::start();
 
 	a.setQuitOnLastWindowClosed(false);
 
-	if (g.s.bFirstTime) {
-		if (QMessageBox::question(g.mw, QLatin1String("Mumble"), MainWindow::tr("This is the first time you're starting Mumble.<br />Would you like to go through the Audio Wizard to configure your soundcard?"), QMessageBox::Yes | QMessageBox::No, QMessageBox::Yes) == QMessageBox::Yes) {
-			AudioWizard *aw = new AudioWizard(g.mw);
-			aw->exec();
-			delete aw;
+	// Configuration updates
+	bool runaudiowizard = false;
+	if (g.s.uiUpdateCounter == 0) {
+		// Previous version was an pre 1.2.3 release or this is the first run
+		runaudiowizard = true;
+
+	} else if (g.s.uiUpdateCounter == 1) {
+		// Previous versions used old idle action style, convert it
+
+		if (g.s.iIdleTime == 5 * 60) { // New default
+			g.s.iaeIdleAction = Settings::Nothing;
+		} else {
+			g.s.iIdleTime = 60 * qRound(g.s.iIdleTime / 60.); // Round to minutes
+			g.s.iaeIdleAction = Settings::Deafen; // Old behavior
 		}
-		g.s.bFirstTime = false;
 	}
 
+	if (runaudiowizard) {
+		AudioWizard *aw = new AudioWizard(g.mw);
+		aw->exec();
+		delete aw;
+	}
+
+	g.s.uiUpdateCounter = 2;
+
 	if (! CertWizard::validateCert(g.s.kpCertificate)) {
 		QDir qd(QDesktopServices::storageLocation(QDesktopServices::DocumentsLocation));
 		QFile qf(qd.absoluteFilePath(QLatin1String("MumbleAutomaticCertificateBackup.p12")));
@@ -389,52 +452,57 @@ int main(int argc, char **argv) {
 	if (QDateTime::currentDateTime().daysTo(g.s.kpCertificate.first.first().expiryDate()) < 14)
 		g.l->log(Log::Warning, CertWizard::tr("<b>Certificate Expiry:</b> Your certificate is about to expire. You need to renew it, or you will no longer be able to connect to servers you are registered on."));
 
+#ifdef QT_NO_DEBUG
+#ifndef SNAPSHOT_BUILD
 	if (g.s.bUpdateCheck)
+#endif
 		new VersionCheck(true, g.mw);
-
 #ifdef SNAPSHOT_BUILD
 	new VersionCheck(false, g.mw, true);
 #endif
-
-	if (g.s.bPluginCheck)
+#else
+	g.mw->msgBox(MainWindow::tr("Skipping version check in debug mode."));
+#endif
+	if (g.s.bPluginCheck) {
 		g.p->checkUpdates();
+	}
 
 	if (url.isValid()) {
 		OpenURLEvent *oue = new OpenURLEvent(url);
 		qApp->postEvent(g.mw, oue);
 #ifdef Q_OS_MAC
-	} else if (os_url) {
-		OpenURLEvent *oue = new OpenURLEvent(QUrl::fromEncoded(QByteArray(os_url)));
+	} else if (! a.quLaunchURL.isEmpty()) {
+		OpenURLEvent *oue = new OpenURLEvent(a.quLaunchURL);
 		qApp->postEvent(g.mw, oue);
 #endif
 	} else {
-		g.mw->on_qaServerConnect_triggered();
+		g.mw->on_qaServerConnect_triggered(true);
 	}
 
-	res=a.exec();
+	if (! g.bQuit)
+		res=a.exec();
 
 	g.s.save();
-	if (g.sh && g.sh->isRunning())
+
+	ServerHandlerPtr sh = g.sh;
+
+	if (sh && sh->isRunning())
 		Database::setShortcuts(g.sh->qbaDigest, g.s.qlShortcuts);
 
-	AudioInputPtr ai = g.ai;
-	AudioOutputPtr ao = g.ao;
+	Audio::stop();
 
-	g.ao.reset();
-	g.ai.reset();
-
-	while (! ai.unique() || ! ao.unique()) {}
-
-	ai.reset();
-	ao.reset();
-
-	g.sh->disconnect();
+	if (sh)
+		sh->disconnect();
 
 	delete mfeh;
 
 	delete srpc;
 
-	delete g.sh;
+	g.sh.reset();
+	while (sh && ! sh.unique())
+		QThread::yieldCurrentThread();
+	sh.reset();
+
 	delete g.mw;
 
 	delete g.nam;
