@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2011, Thorvald Natvig <thorvald@natvig.com>
 
    All rights reserved.
 
@@ -28,6 +28,8 @@
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
+#include "mumble_pch.hpp"
+
 #include <windows.h>
 #include <tlhelp32.h>
 #include <dbghelp.h>
@@ -51,6 +53,9 @@ static MINIDUMP_USER_STREAM musComment;
 
 static int cpuinfo[4];
 
+bool bIsWin7 = false;
+bool bIsVistaSP1 = false;
+
 static void mumbleMessageOutput(QtMsgType type, const char *msg) {
 	char c;
 	switch (type) {
@@ -96,6 +101,60 @@ static LONG WINAPI MumbleUnhandledExceptionFilter(struct _EXCEPTION_POINTERS* Ex
 	return EXCEPTION_CONTINUE_SEARCH;
 }
 
+static void enableCrashOnCrashes() {
+	// Makes sure the application actually crashes when one of its callbacks
+	// called from the kernel crashes.
+	//
+	// See http://support.microsoft.com/kb/976038
+	//     http://www.altdevblogaday.com/2012/07/06/when-even-crashing-doesnt-work/
+
+	typedef BOOL (WINAPI *tGetPolicy)(LPDWORD lpFlags);
+	typedef BOOL (WINAPI *tSetPolicy)(DWORD dwFlags);
+
+	const DWORD PROCESS_CALLBACK_FILTER_ENABLED = 0x01;
+
+	HMODULE kernel32 = LoadLibrary(L"kernel32.dll");
+
+	tGetPolicy pGetPolicy = (tGetPolicy) GetProcAddress(kernel32,
+		"GetProcessUserModeExceptionPolicy");
+
+	tSetPolicy pSetPolicy = (tSetPolicy) GetProcAddress(kernel32,
+		"SetProcessUserModeExceptionPolicy");
+
+	if (pGetPolicy && pSetPolicy) { // Only available as of Vista SP2 / Win7 SP1
+		DWORD dwFlags;
+		if (pGetPolicy(&dwFlags)) {
+			if (!pSetPolicy(dwFlags & ~PROCESS_CALLBACK_FILTER_ENABLED))
+				qWarning("enableCrashOnCrashes: Failed to set policy");
+		} else {
+			qWarning("enableCrashOnCrashes: Failed to get policy");
+		}
+	}
+}
+
+BOOL SetHeapOptions() {
+	HMODULE hLib = LoadLibrary(L"kernel32.dll");
+	if (hLib == NULL)
+		return FALSE;
+
+	typedef BOOL (WINAPI *HSI)(HANDLE, HEAP_INFORMATION_CLASS, PVOID, SIZE_T);
+	HSI pHsi = (HSI)GetProcAddress(hLib, "HeapSetInformation");
+	if (!pHsi) {
+		FreeLibrary(hLib);
+		return FALSE;
+	}
+
+#ifndef HeapEnableTerminationOnCorruption
+#define HeapEnableTerminationOnCorruption (HEAP_INFORMATION_CLASS)1
+#endif
+
+	BOOL fRet = (pHsi)(NULL, HeapEnableTerminationOnCorruption, NULL, 0) ? TRUE : FALSE;
+	if (hLib)
+		FreeLibrary(hLib);
+
+	return fRet;
+}
+
 FARPROC WINAPI delayHook(unsigned dliNotify, PDelayLoadInfo pdli) {
 	if (dliNotify != dliNotePreLoadLibrary)
 		return 0;
@@ -156,35 +215,26 @@ void os_init() {
 		exit(0);
 	}
 
+	OSVERSIONINFOEXW ovi;
+	memset(&ovi, 0, sizeof(ovi));
+
+	ovi.dwOSVersionInfoSize = sizeof(ovi);
+	GetVersionEx(reinterpret_cast<OSVERSIONINFOW *>(&ovi));
+	bIsWin7 = (ovi.dwMajorVersion >= 7) || ((ovi.dwMajorVersion == 6) &&(ovi.dwBuildNumber >= 7100));
+	bIsVistaSP1 = (ovi.dwMajorVersion >= 7) || ((ovi.dwMajorVersion == 6) &&(ovi.dwBuildNumber >= 6001));
+
 	unsigned int currentControl = 0;
 	_controlfp_s(&currentControl, _DN_FLUSH, _MCW_DN);
 
+	SetHeapOptions();
+	enableCrashOnCrashes();
 	mumble_speex_init();
 
 #ifdef QT_NO_DEBUG
-#ifdef COMPAT_CLIENT
-	errno_t res = 0;
-	size_t reqSize, bSize;
-	_wgetenv_s(&reqSize, NULL, 0, L"APPDATA");
-	if (reqSize > 0) {
-		reqSize += strlen("/Mumble/Console11x.txt");
-		bSize = reqSize;
-
-		STACKVAR(wchar_t, buff, reqSize+1);
-
-		_wgetenv_s(&reqSize, buff, bSize, L"APPDATA");
-		wcscat_s(buff, bSize, L"/Mumble/Console11x.txt");
-		res = _wfopen_s(&fConsole, buff, L"a+");
-	}
-	if ((res != 0) || (! fConsole)) {
-		res=_wfopen_s(&fConsole, L"Console11x.txt", L"a+");
-	}
-#else
 	QString console = g.qdBasePath.filePath(QLatin1String("Console.txt"));
-	errno_t res = _wfopen_s(&fConsole, console.utf16(), L"a+");
-#endif
+	fConsole = _wfsopen(console.toStdWString().c_str(), L"a+", _SH_DENYWR);
 
-	if ((res == 0) && fConsole)
+	if (fConsole)
 		qInstallMsgHandler(mumbleMessageOutput);
 
 	QString hash;
@@ -202,26 +252,60 @@ void os_init() {
 
 	QString comment = QString::fromLatin1("%1\n%2\n%3").arg(QString::fromLatin1(MUMBLE_RELEASE), QString::fromLatin1(MUMTEXT(MUMBLE_VERSION_STRING)), hash);
 
-	wcscpy_s(wcComment, PATH_MAX, comment.utf16());
+	wcscpy_s(wcComment, PATH_MAX, comment.toStdWString().c_str());
 	musComment.Type = CommentStreamW;
 	musComment.Buffer = wcComment;
 	musComment.BufferSize = wcslen(wcComment) * sizeof(wchar_t);
 
-#ifdef COMPAT_CLIENT
-	QString dump = QDesktopServices::storageLocation(QDesktopServices::DataLocation) + QLatin1String("\\mumble11x.dmp");
-#else
 	QString dump = g.qdBasePath.filePath(QLatin1String("mumble.dmp"));
-#endif
 
 	QFileInfo fi(dump);
 	QDir::root().mkpath(fi.absolutePath());
 
-	if (wcscpy_s(wcCrashDumpPath, PATH_MAX, dump.utf16()) == 0)
+	if (wcscpy_s(wcCrashDumpPath, PATH_MAX, dump.toStdWString().c_str()) == 0)
 		SetUnhandledExceptionFilter(MumbleUnhandledExceptionFilter);
 
 	// Increase our priority class to live alongside games.
 	if (!SetPriorityClass(GetCurrentProcess(),HIGH_PRIORITY_CLASS))
 		qWarning("Application: Failed to set priority!");
 #endif
+
+	g.qdBasePath.mkpath(QLatin1String("Snapshots"));
+	if (bIsWin7)
+		SetCurrentProcessExplicitAppUserModelID(L"net.sourceforge.mumble.Mumble");
+}
+
+DWORD WinVerifySslCert(const QByteArray& cert) {
+	DWORD errorStatus = -1;
+
+	PCCERT_CONTEXT certContext = CertCreateCertificateContext(X509_ASN_ENCODING, reinterpret_cast<const BYTE*>(cert.constData()), cert.size());
+	if (!certContext) {
+		return errorStatus;
+	}
+
+	LPSTR usage[] = {
+		szOID_PKIX_KP_SERVER_AUTH,
+		szOID_SERVER_GATED_CRYPTO,
+		szOID_SGC_NETSCAPE
+	};
+
+	CERT_CHAIN_PARA chainParameter;
+	memset(&chainParameter, 0, sizeof(CERT_CHAIN_PARA));
+	chainParameter.cbSize = sizeof(CERT_CHAIN_PARA);
+	chainParameter.RequestedUsage.dwType = USAGE_MATCH_TYPE_OR;
+	chainParameter.RequestedUsage.Usage.cUsageIdentifier = ARRAYSIZE(usage);
+	chainParameter.RequestedUsage.Usage.rgpszUsageIdentifier = usage;
+
+	PCCERT_CHAIN_CONTEXT chainContext = NULL;
+	CertGetCertificateChain(NULL, certContext, NULL, NULL, &chainParameter, 0, NULL, &chainContext);
+
+	if (chainContext) {
+		errorStatus = chainContext->TrustStatus.dwErrorStatus;
+		CertFreeCertificateChain(chainContext);
+	}
+
+	CertFreeCertificateContext(certContext);
+
+	return errorStatus;
 }
 
