@@ -1,5 +1,5 @@
-/* Copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
-   Copyright (C) 2009, Stefan Hacker <dd0t@users.sourceforge.net>
+/* Copyright (C) 2005-2011, Thorvald Natvig <thorvald@natvig.com>
+   Copyright (C) 2009-2011, Stefan Hacker <dd0t@users.sourceforge.net>
 
    All rights reserved.
 
@@ -29,14 +29,21 @@
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
-#ifndef _SERVERUSER_H
-#define _SERVERUSER_H
+#ifndef SERVERUSER_H_
+#define SERVERUSER_H_
+
+#include <QtCore/QStringList>
+
+#ifdef Q_OS_UNIX
+#include <sys/socket.h>
+#else
+#include <winsock2.h>
+#endif
 
-#include "murmur_pch.h"
-#include "User.h"
 #include "Connection.h"
 #include "Net.h"
 #include "Timer.h"
+#include "User.h"
 
 // Unfortunately, this needs to be "large enough" to hold
 // enough frames to account for both short-term and
@@ -103,6 +110,7 @@ class ServerUser : public Connection, public User {
 		bool bUdp;
 
 		QList<int> qlCodecs;
+		bool bOpus;
 
 		QStringList qslAccessTokens;
 
@@ -120,9 +128,8 @@ class ServerUser : public Connection, public User {
 #endif
 		BandwidthRecord bwr;
 		struct sockaddr_storage saiUdpAddress;
+		struct sockaddr_storage saiTcpLocalAddress;
 		ServerUser(Server *parent, QSslSocket *socket);
 };
 
-#else
-class ServerUser;
 #endif
